function x0 = analytsp(fsin,f1,f2,snr,m)
% x0 = analytsp(fsin,f1,f2,snr,m)
% Demo fr analytische Signale: Spektren und Dezimation/Interpolation.
% Alle Parameter sind optional. Die Defaultwerte sind:
% fsin=0.1, f1=0.02, f2=0.18, snr=0.4, m=50.
% Es wird ein Sinus-Signal der Frequenz fsin erzeugt und 
% ein weisses Rauschen hinzugefgt. Dieses Signal wird mit 
% einem IIR-Bandpass vom Grad 8 und den Passbandgrenzen f1 und f2 
% gefiltert, so dass ein bandbegrenztes Signal mit Signal/Rauschver-
% hltnis snr entsteht. 
% Anschliessend erfolgt die Wandlung in ein analytisches Signal und 
% danach die bei analytischen Signalen mgliche Dezimation um 2.
% Durch Einfgung von Nullen wird die Abtastrate wieder auf den 
% ursprnglichen Wert gebracht und der Imaginrteil zur Interpolation 
% mit dem Hilbertfilter gefiltert.
% Dargestellt werden auf dem ersten Schirm die Frequenzgnge der Filter
% und die Spektren ausgewhlter Signale. 
% Durch beliebigen Tastendruck gelangt man zum zweiten Schirm, der die  
% Signale im Zeitbereich darstellt: 
% a.  ROT:   das reelle Ausganssignal.
% b.  BLAU:  den Realteil des dezimierten analytischen Signals nach
%            Einfgung von Nullen zur Hochtastung.
% c.  GRN:  die Ausgabe des zur Interpolation des dezimierten und 
%            wieder hochgetasteten Imaginrteils benutzten Hilbertfilters.
% m gibt die Lnge der darzustellenden Sequenz an.
% Einheit von fsin,f2,f2 ist die Nyquistfrequenz = Abtastrate/2;
% das Hilberfilter lsst dabei nur den Bereich 0.02..0.98 zu.
% Fr die Demonstration eignet sich z.B. analytsp(0.1,0.05,0.15,1.5);

% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de
%
% Defaults
% ========
if nargin<1 fsin=0.1; end;
if nargin<2 f1=0.02; end;
if nargin<3 f2=0.18; end;
if nargin<4 snr=0.4; end;
if nargin<5 m=50; end;        % Lnge der Darstellung im Zeitbereich
if (fsin<-1)|(fsin>1) error('fsin nicht im Bereich -1...1'); end;
if f1<0.02 error('f1 darf nicht kleiner sein als 0.02'); end;
if f2>0.98 error('f2 darf nicht grsser sein als 0.98'); end;
if f2-f1 < 0.04 error('f2 - f1 muss grsser als 0.04 sein'); end;
if snr<0 error('snr kann nicht kleiner als 0 sein'); end;
if (abs(fsin)<f1)|(abs(fsin)>f2) hinweis='Das Signal liegt nicht im Bandpass.', end;
%
% Filterentwurf
% =============
ga = 8;                % Filtergrad fr IIR-Bandpass
gh =126;               % Filtergrad fr Hilbertfilter
gh2= gh/2;
[ba,aa]=ellip(ga/2,0.5,60,[f1,f2]); % Bandpassfilterkoeffizienten
% Entwurf eines Hilbertfilters via Halbbandtiefpass:
bh=-2*sign(fsin)*real(specrot(fir1(gh,1/2),pi/2)); % Hibertfilter
% zum Hilbertfilter passende Verzgerung:
bv=[zeros(1,gh/2),1,zeros(1,gh/2)]; % Koeffiz. der Verzgerung
%
% Signalerzeugung
% ===============
xanf = 1300;           % Darstellungsanfang
xend = xanf + m - 1;   % Darstellungende
xe   = xanf + 1023;    % Ende fr die 1024-FFT
n = xanf+max(1023,m)+200; % Sample-Lnge
t = 1:n;               % Zeitbereich in Abtastschritten
z = (1024:-1:1)/512;   % Frequenzbereich 0..Abtastfrequenz entspricht 0..2
sn= 1+(f2-f1)*snr;     % SNR auf Bandbreite umrechnen
x = filter(ba,aa,sinnoise(pi*fsin*t,sn)); % Sinus + Rauschen
y = filter(bv,1,x)+i*filter(bh,1,x);  % analytisches Signal
y2= y(1:2:length(y));  % Dezimation um 2
y3= nullein(y2,1);     % Hochtastung durch Einfgung von Nullen 
y4= filter(bh,1,imag(-y3));  % Rckdrehung mit Hilbertfilter
%
% Signaldarstellung
% =================
% Die Darstellung erfolgt in zwei Bildschirmen nacheinander. Zunchst 
% der Filterfrequenzgang und die Spektren.
switch screen
case 1
      pos{2} = [0 315 263 263];
      pos{3} = [267 315 263 263];
      pos{4} = [533 315 263 263];
      pos{5} = [0 30 263 263];
      pos{6} = [267 30 263 263];
      pos{7} = [533 30 263 263];
      pos{8} = [1 30 800 550];
   case 2
      pos{2} = [2 402 338 346];
      pos{3} = [343 402 338 346];
      pos{4} = [684 402 338 346];
      pos{5} = [2 30 338 346];
      pos{6} = [343 30 338 346];
      pos{7} = [684 30 338 346];
      pos{8} = [1 31 1024 718];
   otherwise error('Bildpositionen fr screen nicht angegeben.');
end;
for k=2:8              % Bilderrahmen kreieren und Properties setzen
   h(k) = fullscreen;
   set(h(k),'position',pos{k});
end;

% Nun die Bilder der Reihe nach fllen

figure(h(4));
set(h(4),'Name','Filter-Frequenzgang');
nn = 256;
ff = ((1:nn)-0.5)/nn;
si = 20*log10(max(1E-20,abs(freqz(ba,aa,ff*1*pi))));
sh = 20*log10(max(1E-20,abs(freqz(bh,1,ff*1*pi))));
for kk = 1:nn
   if isnan(si(kk))  si(kk) = 1E-20; end;
   if isnan(sh(kk))  sh(kk) = 1E-20; end;
end;
plot(ff,si,ff,sh,2-ff,si,'b:',2-ff,sh,'g:');
axis([0,2,-70,10]);
grid('on');
ylabel('Betragsfrequenzgang (dB)');
xlabel(...
'Omega      (blau: Bandpass;  grn: Hilbertfilter;   roter Marker: Signalfrequenz)');
line([abs(fsin),abs(fsin)],[0,0],'Marker','.','Color','r','MarkerSize',20);
drawnow;

figure(h(2)); 
set(h(2),'Name','Spektrum: Input');
plot(z,abs(fft(x(xanf:xe))));
xlabel('Spektrum: Input');

figure(h(3)); 
set(h(3),'Name','Spektrum: reell dezimiert');
plot(z,abs(fft([nullein(x(xanf:2:xe),1),0])));
xlabel('Spektrum: reell dezimiert');

figure(h(5)); 
set(h(5),'Name','Spektrum des analytischen Signals');
plot(z,abs(fft(y(xanf:xe))));
xlabel('Spektrum des analytischen Signals');

figure(h(6)); 
set(h(6),'Name','Spektrum: Realteil dezimiert');
plot(z,abs(fft(y4(xanf:xe))));
xlabel('Spektrum: Realteil dezimiert');

figure(h(7)); 
set(h(7),'Name','Spektrum: Rekonstruktion');
plot(z,abs(fft(real(y3(xanf-gh2:xe-gh2)+y4(xanf:xe)))));
xlabel(' ');
%
% Signaldarstellung im Zeitbereich 
% ================================
figure(h(8));
set(h(8),'Name','Zeitbereich');
plot(1:m,x(xanf-gh:xend-gh),'r-',1:m,x(xanf-gh:xend-gh),'r.',...
   1:m,real(y3(xanf-gh2:xend-gh2)),'b-',1:m,real(y3(xanf-gh2:xend-gh2)),'bo',...
   1:m,real(y4(xanf:xend)),'g-',1:m,real(y4(xanf:xend)),'go');
title('Hochtastung eines analytischen Signals durch Interpolation mit Hilbertfilter');
xlabel('ROT: Inputsignal;  BLAU: Realteil nach Dezimation;  GRN: Interpolation des Imaginrteils');

