function y = audiokomp(tab,r,x,f1,f2,clip)
% audiokomp(lut,r,x,f1,f2,clip)
% Audiokompression fr Sprachsignale der Abtastrate  fs = 8000 Hz
% tab :  LUT
% r   :  Klangfarbenfilter ( r<0 : Tiefenanhebung; r>0 Hhenanhebung) 
% x   :  Eingangssignal
% f1  :  untere Bandpassgrenze [Hz]
% f2  :  obere Bandpassgrenze [Hz]
% clip:  Clipp-Schwelle (0 < clipp <=1)
% y   :  Ausgangssignal
%
% Wird x nicht angegeben, so wird ein Ton von 880 Hz mit in quidistanten 
% Stufen ansteigender Amplitude erzeugt und das Resultat angezeigt.
%
% Prinzip:
% 1. elliptisches Bandpassfilter mit Durchlass ug...og
% 2. Klangfarbfilter whlbar (FIR, g = 5)
% 3. Kompression
% 4. Clippen und Filtern
% Der Absolutwert des Signals (0...1) wird mithilfe einer Tabelle LUT
% gewandelt in ein bersteuerungssignal (>=1), welches anschliessend 
% tiefpassgefiltert wird. Das Eingangssignal wird entsprechend verzgert 
% durch das Tiefpasssignal dividiert:
%
%      ------> Verzgerung ---------
%     |                             |
%   --|                            DIV -->
%     |                             |
%      --> ABS --> LUT --> IIR-TP --


% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de
%
% Konstanten und Defaults
% =======================
fs = 8000;                                  % Abtastrate
fsiz = 12;                                  % Fontgrsse
gh = 10;                                    % Grad des Hilbertfilters
c  = 0.011;                                 % Tiefpassgrenze
gtp = 4;                                    % Tiefpassfiltergrad
if nargin<1  tab = 0; end
lut = tab;
if length(tab)<2
   q   = 5;
   lut = raisedcosine([1,q],50); 
end
if nargin<2  r = 0; end                     % flacher Frequenzgang
if nargin<3
   n = 16384;                               % Inputlnge
   x = cos(2*pi*880/fs*(0:n-1));            % Ton a'
   x = 0.1*x.*round((1:n)/2000);            % Amplitudentreppe
end
if nargin<4  f1 = 100;  end                 % untere Bandpassgrenze 100 Hz
if nargin<5  f2 = 2500; end                 % obere Bandpassgrenze 2500 Hz
if nargin<6 clip= 0.9;  end                 % kaum klippen
n = length(x);                              % Anzahl der Abtastungen
%
% Bandpassfilter
% ==============
ug = f1/fs*2;                               % untere Bandpassgrenze
og = f2/fs*2;                               % obere Bandpassgrenze
[b,a]= ellip(4,1,70,[ug,og]);               % IIR-Filterentwurf               
delb = grpdelay(b,a,ug:0.1:og,2);           % Gruppenlaufzeit
y1   = filter(b,a,x);                       % Filtern
y1   = y1/max(abs(y1));                     % Normierung
%
% Klangfarbenfilter
% =================
bk = 0.5*(1+r)*[0 0 1 0 0] - r*fir1(4,0.2); % FIR-Filterentwurf
bk = bk/sum(bk);                            % Normierung
y2 = filter(bk,1,y1);                       % Filtern
y2 = y2/max(abs(y2));                       % Normierung
%
% Kompression
% ===========
bh  = remez(gh,[ug,og],[1 1],'Hilbert');    % Hilbertfilterentwurf
y2a = delay(y2,gh/2)+i*filter(bh,1,y2);     % Hilbertfilter
y2a = y2a/max(abs(y2a));                    % Normierung
y3  = lut(ceil((length(lut)-1)*(abs(y2a)+eps)));
[bb,aa] = butter(gtp,c);                    % Tiefpassfilterentwurf
del = round(mean(grpdelay(bb,aa,0:c/10:c,2)));
y4 = abs(filter(bb,aa,y3)+eps);             % IIR-Tiefpass
y6 = delay(y2,del)./y4;                     % Dynamik-Kompression
%
% Clippen
% =======
mm = clip*max(abs(y6));                     % Clipp-Schwelle
y7 = max(-mm,min(mm,y6));                   % Clippen
y8 = filter(b,a,y7);                        % Filtern
%
% Ausgabe
% =======
y  = y8/max(abs(y8));                       % Normierung
%
if nargout>0  break; end                    % keine Graphik, kein Audio
%
% Darstellung
% ===========
fh1 = fullscreen('Gesamtfilter','w',0,[2.5 2; 1 1]);
filterplot({{b;a};bk},0,1,0,-80,15,1,16,1)
title('Bandpass plus Klangfarbfilter','FontSize',fsiz)

fh2 = fullscreen('LUT','w',0,[2.5 2; 1 2]);
plot((0:length(lut)-1)/length(lut),lut)

fh3 = fullscreen('Signale','w',2,[1.8 2; 1.8 1]);
oh  = plot(1:n,[delay(y2,del);delay(y3,del);y4;y8]);
set(oh(3),'LineWidth',2)
set(oh(4),'Color','m')
line([0 0; n n],max(abs(delay(y2,del)))*[-1 +1;-1 +1],'LineStyle',':','Color','b')
line([0 0; n n],max(abs(y8))*[-1 +1;-1 +1],'LineStyle',':','Color','m')
legend('Bandpasssignal','LUT-Signal','Divisor','Ausgangssignal')

fh4 = fullscreen('Ortsdiagramm','w',2,[1.8 2; 1.8 2]);
axes('Position',[0.06 0.09 0.88 0.82]) 
y9 = delay(y2a,del);
plot(abs(y9),abs(y9./y4))
title('Kompession ohne Clippen','FontSize',13)
xlabel('Amplitude des Inputsignals nach dem Klangfarbenfilter','FontSize',13)
ylabel('Amplitude des Outputsignals vor dem Clippen','FontSize',13)
axis([0 1 0 0.6]);
%
% Audioausgabe
% ============
soundsc(x,fs);  pause(2)  % Inputsignal
soundsc(y2,fs); pause(2)  % nach Bandpass- und Klangfarbenfilter
soundsc(y,fs);  pause(2)  % Outputsignal

