function y = audiokomp2(tab,r,x,f1,f2,clip)
% audiokomp2(lut,r,x,f1,f2,clip)
%
% Audiokompression fr Sprachsignale der Abtastrate  fs = 8000 Hz
% tab :  Tabellenwerte (>= 1.0) zur Umsetzung von Absolut-Amplituden in
%        Faktoren zur Korrektur. Der Index ist ceil(1+10*abs(x)), 
%        Signale im Bereich -1<x<+1 werden also auf die ersten 10 Tabellenwerte
%        abgebildet, z.b. auf lut = [1 1 1 1 1 1 1 1.1 1.2 1.3 ...]
% r   :  Klangfarbenfilter ( -1<r<0 : Tiefenanhebung; 0<r<1 Hhenanhebung) 
% x   :  Eingangssignal
% f1  :  untere Bandpassgrenze [Hz]
% f2  :  obere Bandpassgrenze [Hz]
% clip:  Clipp-Schwelle (0 < clip <=1)
% y   :  Ausgangssignal
%
% Wird x nicht angegeben, so wird ein Ton von 880 Hz mit in quidistanten 
% Stufen ansteigender Amplitude erzeugt und das Resultat angezeigt.
%
% Prinzip:
% 1. elliptisches Bandpassfilter mit Durchlass ug...og
% 2. Klangfarbfilter whlbar (FIR, g = 5)
% 3. Das analytische Signal wird in Stufen von pi/8 gedreht und der Realteil 
%    der kleinsten Amplitude in Abschnitten der Lnge 2*round(3.5*fs/f1) ausgewhlt.
% 4. Klassische Kompression:
%    Der Absolutwert des Signals nach 3. wird mithilfe einer Tabelle LUT
%    gewandelt in ein bersteuerungssignal (>=1), welches anschliessend 
%    tiefpassgefiltert wird. Das Eingangssignal wird entsprechend verzgert 
%    durch das Tiefpasssignal dividiert:
%
%      ------> Verzgerung ---------
%     |                             |
%   --|                            DIV -->
%     |                             |
%      --> ABS --> LUT --> IIR-TP --


% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de
%
% Konstanten und Defaults
% =======================
fs = 8000;                                  % Abtastrate
fsiz = 12;                                  % Fontgrsse
gh = 10;                                    % Grad des Hilbertfilters
c  = 0.011;                                 % Tiefpassgrenze
gtp= 4;                                     % Tiefpassfiltergrad
q  = 8;
if nargin<1  tab = 0; end
lut = tab;
if length(tab)<2
   lut = [ones(1,7) (1.1:0.1:100)]; 
end
if length(lut)<1000  lut = [lut lut(end)*ones(1,1000-length(lut))]; end
if nargin<2  r = 0; end                     % flacher Frequenzgang
if nargin<3
   n = 16000;                               % Inputlnge
   x = cos(2*pi*880/fs*(0:n-1));            % Ton a'
   x = 0.3*x.*round((1:n)/2000);            % Amplitudentreppe
end
if nargin<4  f1 = 300;  end                 % untere Bandpassgrenze 300 Hz
if nargin<5  f2 = 2400; end                 % obere Bandpassgrenze 2400 Hz
m  = 2*round(3.5*fs/f1);                    % Anzahl der Abtastungen fr Mittelwert
k  = mod(length(x),m);
if k>0  x  = [x, zeros(1,m-k)]; end         % Zero-Padding
if nargin<6 clip= 0.9;  end                 % etwas klippen
n  = length(x);                             % Anzahl der Abtastungen
%
% Bandpassfilter
% ==============
ug = f1/fs*2;                               % untere Bandpassgrenze
og = f2/fs*2;                               % obere Bandpassgrenze
[b,a]= ellip(4,1,70,[ug,og]);               % IIR-Filterentwurf               
delb = grpdelay(b,a,ug:0.1:og,2);           % Gruppenlaufzeit
y1   = filter(b,a,x);                       % Filtern
%
% Klangfarbenfilter
% =================
bk = 0.5*(1+r)*[0 0 1 0 0] - r*fir1(4,0.2); % FIR-Filterentwurf
bk = bk/sum(bk);                            % Normierung
y2 = filter(bk,1,y1);                       % Filtern
%
% Kompression
% ===========
bh  = remez(gh,[ug,og],[1 1],'Hilbert');    % Hilbertfilterentwurf
y2a = delay(y2,gh/2)+i*filter(bh,1,y2);     % Hilbertfilter

y20 = real(y2a);              
y21 = real(y2a*exp(1*i*pi/q));              % Phasenverschiebung
y22 = real(y2a*exp(2*i*pi/q));              % Phasenverschiebung
y23 = real(y2a*exp(3*i*pi/q));              % Phasenverschiebung
y24 = real(y2a*exp(4*i*pi/q));              % Phasenverschiebung
y25 = real(y2a*exp(5*i*pi/q));              % Phasenverschiebung
y26 = real(y2a*exp(6*i*pi/q));              % Phasenverschiebung
y27 = real(y2a*exp(7*i*pi/q));              % Phasenverschiebung

mx(1,:) = max(abs(reshape(y20,m,n/m)));     % Amplitude
mx(2,:) = max(abs(reshape(y21,m,n/m)));     % Amplitude
mx(3,:) = max(abs(reshape(y22,m,n/m)));     % Amplitude
mx(4,:) = max(abs(reshape(y23,m,n/m)));     % Amplitude
mx(5,:) = max(abs(reshape(y24,m,n/m)));     % Amplitude
mx(6,:) = max(abs(reshape(y25,m,n/m)));     % Amplitude
mx(7,:) = max(abs(reshape(y26,m,n/m)));     % Amplitude
mx(8,:) = max(abs(reshape(y27,m,n/m)));     % Amplitude

mmx = min(mx);
ix  = 1*(mx(2,:)==mmx) + 2*(mx(3,:)==mmx) + 3*(mx(4,:)==mmx) + ...
      4*(mx(5,:)==mmx) + 5*(mx(6,:)==mmx) + 6*(mx(7,:)==mmx) + 7*(mx(8,:)==mmx);
ax = ix(1)*ones(1,m/2);   
for k = 1:length(ix)-1
   dx = ix(k+1)-ix(k);
   if dx>+q/2  ix(k+1:end) = ix(k+1:end) - q; end
   if dx<-q/2  ix(k+1:end) = ix(k+1:end) + q; end
   ax = [ax, raisedcosine(ix(k:k+1),m)];      
end
ax = [ax, ix(end)*ones(1,m/2)];
if length(ax)<length(y2a) ax = [ax, ax(end)*ones(1,length(y2a)-length(ax))]; end

y2b = y2a.*exp(ax*i*pi/q);                  % optimale Phasenverschiebung
y3  = lut(ceil(10*(abs(y2b))+eps));
[bb,aa] = butter(gtp,c);                    % Tiefpassfilterentwurf
del = round(mean(grpdelay(bb,aa,0:c/10:c,2)));
y4 = abs(filter(bb,aa,y3)+eps);             % IIR-Tiefpass
y6 = delay(real(y2b),del)./y4;              % Dynamik-Kompression
%
% Clippen
% =======
mm = clip*max(abs(y6));                     % Clipp-Schwelle
y7 = max(-mm,min(mm,y6));                   % Clippen
y8 = filter(b,a,y7);                        % Filtern
%
% Ausgabe
% =======
y  = y8;
%
if nargout>0  break; end                    % keine Graphik, kein Audio
%
% Darstellung
% ===========
fh1 = fullscreen('Phase','w',0,[2.5 3; 1 1]);
plot(ax)

fh2 = fullscreen('Signal ohne/mit Phasendrehung','w',2,[1.8 3; 1.8 1]);
plot(1:length(y2a),abs(real(y2a)),'b',1:length(y2b),abs(real(y2b)),'r')

fh3 = fullscreen('Gesamtfilter','w',0,[2.5 3; 1 2]);
filterplot({{b;a};bk},0,1,0,-80,15+30*(r>0.7)*r^4,1,16,1)
title('Bandpass plus Klangfarbfilter','FontSize',fsiz)

fh4 = fullscreen('LUT','w',0,[2.5 3; 1 3]);
plot((0:length(lut)-1)/10,lut)
set(gca,'XLim',[0 3])
xlabel('Inputamplitude','FontSize',13)

fh5 = fullscreen('Signale','w',2,[1.8 3; 1.8 2]);
oh  = plot(1:n,[delay(y2,del);delay(y3,del);y4;y8]);
set(oh(3),'LineWidth',2)
set(oh(4),'Color','m')
line([0 0; n n],max(abs(delay(y2,del)))*[-1 +1;-1 +1],'LineStyle',':','Color','b')
line([0 0; n n],max(abs(y6))*[-1 +1;-1 +1],'LineStyle',':','Color','m')
legend('Bandpasssignal','LUT-Signal','Divisor','Ausgangssignal')

fh6 = fullscreen('Ortsdiagramm','w',2,[1.8 3; 1.8 3]);
axes('Position',[0.06 0.09 0.88 0.82])
my = max(abs(y2a));
plot(abs(delay(y2a,del))/my,abs(delay(y2b,del)./y4)/my)
title('Kompession ohne Clippen','FontSize',13)
xlabel('Amplitude des Inputsignals nach dem Klangfarbenfilter','FontSize',13)
ylabel('Amplitude des Outputsignals vor dem Clippen','FontSize',13)
axis([0 1 0 0.6]);
%
% Audioausgabe
% ============
soundsc(x, fs); pause(2)
soundsc(y2,fs); pause(2)
soundsc(y, fs); pause(2)

