function [y,fr,snr,sm] = ddspoly(f,pk,n,m,pt,po,adith,window,p0)
% [y,fr,snr,sm] = ddspoly(f,pk,n,m,pt,po,adith,window,p0)
% Simulation einer DDS mit Berechnung der Sinuswerte durch ein Polynom.
%  f   :  Frequenz relativ zur Abtastfrequenz
%  pk  :  Polynomkoeffizienten (wie bei polyfit und polyval)
%         alternativ kann der Polynomgrad angegeben werden.
%  n   :  Anzahl der Bits des Phasenakkumulators
%  m   :  Anzahl der Bits des Sinus-Arguments
%  pt  :  Anzahl der Datenbits der Arithmetik im 2-Komplement
%  po  :  Anzahl der Datenbits der Ausgabe im 2-Komplement
%  adith: zum Ausgangssignal wird vor der Quantisierung auf po
%         Bits das Dithersignal adith*(rand-0.5)*2^(1-po) addiert
%         abs(adith)<1 verhindert das dithern. Sinnvoll ist adith = 1
% window: 0 Rechteck;  1 Hanning;  2 Hamming;  3 Kaiser  (Default 0)
%  p0  :  Phasenwinkel des ersten Wertes
%
%  y   :  eine Matrix, die in vier Zeilen je maximal
%         4096 Abtastschritte des erzeugten Sinus enthltritten Grades
%  fr  :  tatschlich realisierte Frequenz
%  snr :  Signal/Rauschverhltniss
%  sm  :  minimale Unterdrckung von Nebenfrequenzen in dB
%
% Es wird das Spektrum des berechneten Signals sowie des korrekten Sinus 
% gezeigt. Die Linienstrke der einzelnen Kurven kann mit den 
% Zifferntasten 1...4 getoggelt werden.
%
% Wird die Ausgabe einer Variablen zugewiesen, so gibt es 
% keine Grafik.

% Autor: Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<9     p0 = 0;  end
if nargin<8 window = 3;  end
if nargin<7  adith = 0;  end
if nargin<6     po = 24; end
if nargin<5     pt = 24; end
if nargin<4      m = 24; end
if nargin<3      n = 48; end
if nargin<2     pk = 8;  end
if nargin<1 f = 0.4*rand;end

% Konstanten
% ==========
if length(pk)>1
   g = length(pk) - 1;
   pkneu = 0;
else
   g = pk;
   pk = polynomfit('sin(pi/2*x)',g,pt,1024);
   pkneu = 1;
end
if screen==1  
   pos = [2 30 525 548; 532 33 267 546];
   fsiz = 10;                      % FontSize
else
   pos = [4 31 694 716; 703 33 319 714];
   fsiz = 13;                      % FontSize
end

pn = round(p0/(2*pi)*2^n);         % Anfangsphase
pd = round((2^n)*f);               % Phasenschritt
fr = pd*2^-n;                      % realisierte Frequenz
p  = 1/fr;                         % Periode in Anzahl Abtastungen / Schwingung
ks = ceil(fr*4096):floor(fr*16384);% Anzahl der Schwingungen
d  = abs(ks*p - round(ks*p));      % Samplefehler der periodischen Fortsetzung
md = min(d);                       % Minimalfehler
k  = round(p*ks(min(find(d==md))));% Anzahl der Abtastpunkte
k2 = floor(k/2);                   % davon die Hlfte
fm = round(fr*k);                  % Index der Spitze im Spektrum
t  = 0:k-1;                        % Zeitbereich
nf = 6.02*po + 50;                 % Rauschebene in dB
pb = round(0.01*k);

switch window                      % Berechnung des gewhlten Fensters
   case 0,  win = ones(1,k);
   case 1,  win = hanning(k)';
   case 2,  win = hamming(k)';
   case 3,  win = kaiser(k,0.1102*(nf-8.7))';
end   
swin= sum(win);

%
% Fehlerfreier Sinus
% ==================
sc = sin(2*pi*fr*t+p0)*(1-2^(1-pt)); % fehlerfreier Sinus
fc = abs(fft(sc.*win));            % FFT
mfc= max(fc);                      % Normierungsfaktor
ssc= sum(sc.^2);                   % Leistung
r0 = 20*log10(abs(fc(1:k2)/mfc)+eps); % Referenzspektrum in dB
%
% Reihen-Entwicklung g-ten Grades
% ===============================
pa = mod(pn + pd*t,2^n);           % Reprsentation der quidistanten Winkel
xx = pa*2^(2-n);                   % 4/pi * Winkel 
xx = (xx<=2).*xx + (xx>2).*(xx - 4); %
xx = (xx<-1).*(-2 - xx) + (abs(xx)<=1).*xx + (xx>+1).*(2 - xx);
s  = sign(xx).*polyval(pk,abs(xx));% Entwicklung
s  = round(s*2^(po-1))*2^(1-po);   % Quantisierung auf po Bits
s2 = min(1-2^(1-po),max(-1,s));    % Clippen
f2 = fft(s2.*win)/mfc;             % FFT
l2 = 20*log10(abs(f2(1:k2))+eps);  % Spektrum in dB
ds2 = s2 - sc;                     % Fehler der Ausgabe
snr = 10*log10(sum(ds2.^2)/ssc);   % SNR
es2 = fft(real(ds2).*win)/mfc;     % Fehlerspektrum
e02 = 20*log10(abs(es2(1:k2))+eps);% Spektrum in dB
sm  = max(e02([1:max(0,fm-pb),fm+pb:k/2])); % Maximum des Nebenfrequenzspektrums
smf = (find(e02==sm)-1)/k;         % Frequenz dieses Maximums
smf = smf(1);                      % nur eines ausgeben

% Ausgabe
% =======
nn = min([4096 k]);
y = [sc(1:nn); s2(1:nn)];
if nargout>0  break; end
%
% Darstellung
% ===========
Kontrollausgabe = sprintf('ddspoly\n#Abtastpunkte:   %5d\nSollfrequenz:       %15.12f\nFrequenzfehler:     %15.12f\nNebenmaximum bei:   %15.12f\nBetrag:           %6.1f dB\nSNR:              %6.1f dB\nPolynomgrad:        %2d',k,f,fr-f,smf,sm,snr,g)
ug = max([-250  min([sm  -6*po])-30]);

if pkneu
   x  = (0:n-1)/(n-1);             % quidistante Punkte
   sn = sin(pi/2*x);               % zugehrige Funktionswerte
   ee = abs(sn - polyval(pk,x));   % Fehler 
   eem= max(ee);
   fh3 = fullscreen('Fehler der Polynomentwicklung');
   plot(pi/2*x,ee)
   title(sprintf('Fehler der Polynomentwicklung vom Grad %2d fr sin(x)',g),'FontSize',fsiz)
   xlabel('x','FontSize',fsiz)
   axis([0  pi/2  0.01*eem  1.2*eem])
end

fh1 = fullscreen('Reihenentwicklung');
lh1 = plot((0:k2-1)/k,[l2;e02;r0]');   % Spektrum
axis([0 1/2 ug 5])
ax = axis;
lh1(4) = line([f f],[ax(3) 0],'Color','m');
set(gcf,'Position',pos(1,:))
set(gca,'Position',[0.08 0.07 0.85 0.87])
legend('DDS','Fehler','Referenz','ideal')
title(sprintf('Reihenentwicklung  g = %2d    f = %7.6f    pt = %2d    po = %2d    SNR = %5.2f',g,f,pt,po,snr),'FontSize',fsiz)
xlabel('Frequenz / Abtastfrequenz','FontSize',fsiz)
ylabel('Spektrum in dB','FontSize',fsiz)
line([0 0.5],sm*[1 1],'LineStyle',':','Color','r')
text(0.505,sm,sprintf('%3.2f dB',sm),'FontSize',fsiz-2,'Color','r')
set(lh1(1),'UserData',10*fh1+1)
set(lh1(2),'UserData',10*fh1+2)
set(lh1(3),'UserData',10*fh1+3)
set(lh1(4),'UserData',10*fh1+4)
set(fh1,'KeyPressFcn','sichtbarkeit');
set(lh1(2),'visible','off')
fh2 = fullscreen('Ausschnitt Taylor-Entwicklung');
lh2 = plot((0:k2-1)/k,[l2;e02;r0]');   % Spektrum
axis([f-0.01  f+0.01  ug  5])
ax = axis;
lh2(4) = line([f f],[ax(3) 0],'Color','m');
set(gcf,'Position',pos(2,:))
set(gca,'Position',[0.13 0.07 0.775 0.87])
legend('DDS','Fehler','Referenz','ideal')
title('Ausschnitt','FontSize',fsiz+2)
set(lh2(1),'UserData',10*fh2+1)
set(lh2(2),'UserData',10*fh2+2)
set(lh2(3),'UserData',10*fh2+3)
set(lh2(4),'UserData',10*fh2+4)
set(fh2,'KeyPressFcn','sichtbarkeit');
set(lh2(2),'visible','off')
