function [y,fr,snr,sm] = ddssim(f,n,m,pt,po,pdith,adith,window,p0)
% [y,fr,snr,sm] = ddssim(f,n,m,pt,po,pdith,adith,window,p0)
% Simulation einer DDS mit folgenden Parametern:
%  f   :  Frequenz relativ zur Abtastfrequenz
%  n   :  Anzahl der Bits des Phasenakkumulators
%  m   :  Anzahl der Adressbits der Sinustabelle
%  pt  :  Wortbreite des ROM
%  po  :  Anzahl der Datenbits der Ausgabe im 2-Komplement
%  pdith: wirkt nur bei der Methode 2: nchster unterer Tabellenwert:
%         Zur ROM-Adresse wird vor der Quantisierung auf m
%         Bits das Dithersignal  pdith*(rand-0.5) addiert
%         abs(pdith)<1 verhindert das dithern. Sinnvoll ist pdith = 1
%  adith: zum Ausgangssignal wird vor der Quantisierung auf po
%         Bits das Dithersignal adith*(rand-0.5)*2^(1-po) addiert
%         abs(adith)<1 verhindert das dithern. Sinnvoll ist adith = 1
% window: 0 Rechteck;  1 Hanning;  2 Hamming;  3 Kaiser  (Default 0)
% p0 :    Phasenwinkel des ersten Wertes
%
%  y   :  eine Matrix, die in vier Zeilen je maximal
%         4096 Abtastschritte des erzeugten Sinus 
%         enthlt fr 4 Verfahren:
%         1. korrekter Sinus (nicht auf p Bits beschrnkt)
%         2. nchster unterer Tabellenwert
%         3. lineare Interpolation
%         4. Taylorentwicklung dritten Grades
%  fr  :  tatschlich realisierte Frequenz
%  snr :  Vektor der Signal/Rauschverhltnisse fr 
%         die vier Zeilen von y
%  sm  :  Vektor der minimalen Unterdrckung von 
%         Nebenfrequenzen in dB fr die vier Zeilen von y
%
% Es wird das Spektrum fr die drei Verfahren der 
% Berechnung des Sinus aus der Tabelle sowie der 
% korrekte Sinus gezeigt. Die Linienstrke der einzelnen 
% Kurven kann mit den Zifferntasten 1...4 getoggelt werden.
%
% Wird die Ausgabe einer Variablen zugewiesen, so gibt es 
% keine Grafik.

% Autor: Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<9     p0 = 0;  end
if nargin<8 window = 0;  end
if nargin<7  adith = 0;  end
if nargin<6  pdith = 0;  end
if nargin<4     pt = 16; end
if nargin<5     po = pt; end
if nargin<3      m = 8;  end
if nargin<2      n = 32; end
if nargin<1 f = 0.4*rand;end

% Konstanten
% ==========
fsiz = 13;                         % FontSize
if screen==1  
   pos = [2 30 525 548; 532 33 267 546];
   fsiz = 10;                      % FontSize
else
   pos = [4 31 694 716; 703 33 319 714];
   fsiz = 13;                      % FontSize
end

pn = round(p0/(2*pi)*2^n);         % Anfangsphase
pd = round((2^n)*f);               % Phasenschritt
fr = pd*2^-n;                      % realisierte Frequenz
p  = 1/fr;                         % Periode in Anzahl Abtastungen / Schwingung
ks = ceil(fr*4096):floor(fr*16384);% Anzahl der Schwingungen
d  = abs(ks*p - round(ks*p));      % Samplefehler der periodischen Fortsetzung
md = min(d);                       % Minimalfehler
k  = round(p*ks(min(find(d==md))));% Anzahl der Abtastpunkte
k2 = floor(k/2);                   % davon die Hlfte
fm = round(fr*k);                  % Index der Spitze im Spektrum
t  = 0:k-1;                        % Zeitbereich
nf = 6.02*po + 50;                 % Rauschebene in dB
pb = round(0.01*k);

switch window                      % Berechnung des gewhlten Fensters
   case 0,  win = ones(1,k);
   case 1,  win = hanning(k)';
   case 2,  win = hamming(k)';
   case 3,  win = kaiser(k,0.1102*(nf-8.7))';
end   
swin= sum(win);
sintab = round(sin(2*pi*(0:2^m-1)*2^-m)*(1-2^(1-pt))*2^(pt-1))*2^(1-pt); % Sinustabelle
%
% Fehlerfreier Sinus
% ==================
sc = sin(2*pi*fr*t+p0)*(1-2^(1-pt)); % fehlerfreier Sinus
fc = abs(fft(sc.*win));            % FFT
mfc= max(fc);                      % Normierungsfaktor
ssc= sum(sc.^2);                   % Leistung
snr(1) = -inf;                     % SNR
sm(1)  = -inf;                     % Maximum des Nebenfrequenzspektrums
r0 = 20*log10(abs(fc(1:k2)/mfc)+eps); % Referenzspektrum in dB
%
% Nchster unterer Tabellenwert
% =============================
pa = pn + pd*t + pdith*(rand(size(t))-0.5)*2^(n-m); % Dithern
ra = floor(mod(floor(pa),2^n)*2^(m-n)) + 1;% ROM-Adresse
s  = sintab(ra);                   % LUT
s  = s + adith*(rand(size(s)) - 0.5)*2^(1-po); % dithern 
s  = round(s*2^(po-1))*2^(1-po);   % Quantisierung auf po Bits
s0 = min(1-2^(1-po),max(-1,s));    % Clippen
f0 = fft(s0.*win)/mfc;             % FFT
l0 = 20*log10(abs(f0(1:k2))+eps);  % Spektrum in dB
ds0 = s0 - sc;                     % Fehler der Ausgabe
snr(2)= 10*log10(sum(ds0.^2)/ssc); % SNR
es0 = fft(real(ds0).*win)/mfc;     % Fehlerspektrum
e00 = 20*log10(abs(es0(1:k2))+eps);% Spektrum in dB
sm(2) = max(e00([1:fm-pb,fm+pb:k/2])); % Maximum des Nebenfrequenzspektrums
%
% Lineare Interpolation
% =====================
p1 = floor(mod(pn+pd*t,2^n)*2^(m-n)+1); % untere Tabellenadresse
dx = mod(pn+pd*t,2^(n-m))*2^(m-n);
s  = (1-dx).*sintab(p1) + dx.*sintab(mod(p1,2^m)+1);
s  = s + adith*(rand(size(s)) - 0.5)*2^(1-po); % dithern 
s  = round(s*2^(po-1))*2^(1-po);   % Quantisierung auf po Bits
s1 = min(1-2^(1-po),max(-1,s));    % Clippen
f1 = fft(s1.*win)/mfc;             % FFT
l1 = 20*log10(abs(f1(1:k2))+eps);  % Spektrum in dB
ds1 = s1 - sc;                     % Fehler der Ausgabe
snr(3) = 10*log10(sum(ds1.^2)/ssc);% SNR
es1 = fft(real(ds1).*win)/mfc;     % Fehlerspektrum
e01 = 20*log10(abs(es1(1:k2))+eps);% Spektrum in dB
sm(3) = max(e01([1:fm-pb,fm+pb:k/2])); % Maximum des Nebenfrequenzspektrums
%
% Taylor-Entwicklung dritten Grades
% =================================
p1 = floor(mod(pn+pd*t,2^n)*2^(m-n)+1); % untere Tabellenadresse
dx = 2*pi*mod(pn+pd*t,2^(n-m))/2^n;
sn = sintab(p1);                   % sin aus Tabelle
cs = sintab(mod(p1+2^(m-2)-1,2^m)+1); % cos aus Tabelle
s  = sn + dx.*(cs - dx.*(sn/2 + dx.*cs/6)); % Taylorentwicklung
s  = s + adith*(rand(size(s)) - 0.5)*2^(1-po); % dithern 
s  = round(s*2^(po-1))*2^(1-po);   % Quantisierung auf p Bits
s2 = min(1-2^(1-po),max(-1,s));    % Clippen
f2 = fft(s2.*win)/mfc;             % FFT
l2 = 20*log10(abs(f2(1:k2))+eps);  % Spektrum in dB
ds2 = s2 - sc;                     % Fehler der Ausgabe
snr(4) = 10*log10(sum(ds2.^2)/ssc);% SNR
es2 = fft(real(ds2).*win)/mfc;     % Fehlerspektrum
e02 = 20*log10(abs(es2(1:k2))+eps);% Spektrum in dB
sm(4) = max(e02([1:fm-pb,fm+pb:k/2])); % Maximum des Nebenfrequenzspektrums
smf = (find(e02==sm(4))-1)/k;      % Frequenz dieses Maximums
smf = smf(1);                      % nur eines ausgeben

% Ausgabe
% =======
nn = min([4096 k]);
y = [sc(1:nn); s0(1:nn); s1(1:nn); s2(1:nn)];
if nargout>0  break; end
%
% Darstellung
% ===========
Kontrollausgabe = sprintf('ddssim (Taylor)\n#Abtastpunkte:   %5d\nSollfrequenz:       %15.12f\nFrequenzfehler:     %15.12f\nNebenmaximum bei:   %15.12f\nBetrag:           %6.1f dB\nSNR:              %6.1f dB',k,f,fr-f,smf,sm(4),snr(4))
ug = max([-250  min([sm(4)  -6*po])-30]);
fh11 = fullscreen('Tabelle direkt');
lh11 = plot((0:k2-1)/k,[l0;e00;r0]');   % Spektrum
axis([0 1/2 ug 5])
lh11(4) = line([f f],[ug 0],'Color','m');
set(gcf,'Position',pos(1,:))
set(gca,'Position',[0.08 0.07 0.85 0.87])
legend('DDS','Fehler','Referenz','ideal')
title(sprintf('Tabelle direkt:    f = %7.6f    n = %2d    %5dX%2d-ROM    po = %2d    SNR = %5.2f',f,n,2^m,pt,po,snr(2)),'FontSize',fsiz)
xlabel('Frequenz / Abtastfrequenz','FontSize',fsiz)
ylabel('Spektrum in dB','FontSize',fsiz)
line([0 0.5],sm(2)*[1 1],'LineStyle',':','Color','r')
text(0.505,sm(2),sprintf('%3.2f dB',sm(2)),'FontSize',fsiz-2,'Color','r')
set(lh11(1),'UserData',10*fh11+1)
set(lh11(2),'UserData',10*fh11+2)
set(lh11(3),'UserData',10*fh11+3)
set(lh11(4),'UserData',10*fh11+4)
set(fh11,'KeyPressFcn','sichtbarkeit');
set(lh11(2),'visible','off')
fh12 = fullscreen('Ausschnitt Tabelle direkt');
lh12 = plot((0:k2-1)/k,[l0;e00;r0]');   % Spektrum
axis([f-0.01  f+0.01  ug  5])
ax = axis;
lh12(4) = line([f f],[ax(3) 0],'Color','m');
set(gcf,'Position',pos(2,:))
set(gca,'Position',[0.13 0.07 0.775 0.87])
legend('DDS','Fehler','Referenz','ideal')
title('Ausschnitt','FontSize',fsiz+2)
set(lh12(1),'UserData',10*fh12+1)
set(lh12(2),'UserData',10*fh12+2)
set(lh12(3),'UserData',10*fh12+3)
set(lh12(4),'UserData',10*fh12+4)
set(fh12,'KeyPressFcn','sichtbarkeit');
set(lh12(2),'visible','off')

fh21 = fullscreen('lineare Interpolation');
lh21 = plot((0:k2-1)/k,[l1;e01;r0]');   % Spektrum
axis([0 1/2 ug 5])
ax = axis;
lh21(4) = line([f f],[ax(3) 0],'Color','m');
set(gcf,'Position',pos(1,:))
set(gca,'Position',[0.08 0.07 0.85 0.87])
legend('DDS','Fehler','Referenz','ideal')
title(sprintf('Lineare Interpolation    f = %7.6f    n = %2d    %5dX%2d-ROM    po = %2d    SNR = %5.2f',f,n,2^m,pt,po,snr(3)),'FontSize',fsiz)
xlabel('Frequenz / Abtastfrequenz','FontSize',fsiz)
ylabel('Spektrum in dB','FontSize',fsiz)
line([0 0.5],sm(3)*[1 1],'LineStyle',':','Color','r')
text(0.505,sm(3),sprintf('%3.2f dB',sm(3)),'FontSize',fsiz-2,'Color','r')
set(lh21(1),'UserData',10*fh21+1)
set(lh21(2),'UserData',10*fh21+2)
set(lh21(3),'UserData',10*fh21+3)
set(lh21(4),'UserData',10*fh21+4)
set(fh21,'KeyPressFcn','sichtbarkeit');
set(lh21(2),'visible','off')
fh22 = fullscreen('Ausschnitt lineare Interpolation');
lh22 = plot((0:k2-1)/k,[l1;e01;r0]');   % Spektrum
axis([f-0.01  f+0.01  ug  5])
ax = axis;
lh22(4) = line([f f],[ax(3) 0],'Color','m');
set(gcf,'Position',pos(2,:))
set(gca,'Position',[0.13 0.07 0.775 0.87])
legend('DDS','Fehler','Referenz','ideal')
title('Ausschnitt','FontSize',fsiz+2)
set(lh22(1),'UserData',10*fh22+1)
set(lh22(2),'UserData',10*fh22+2)
set(lh22(3),'UserData',10*fh22+3)
set(lh22(4),'UserData',10*fh22+4)
set(fh22,'KeyPressFcn','sichtbarkeit');
set(lh22(2),'visible','off')

fh31 = fullscreen('Taylorentwicklung vom Grad 3');
lh31 = plot((0:k2-1)/k,[l2;e02;r0]');   % Spektrum
axis([0 1/2 ug 5])
ax = axis;
lh31(4) = line([f f],[ax(3) 0],'Color','m');
set(gcf,'Position',pos(1,:))
set(gca,'Position',[0.08 0.07 0.85 0.87])
legend('DDS','Fehler','Referenz','ideal')
title(sprintf('Taylor-Entw.    f = %7.6f    n = %2d    %5dX%2d-ROM    po = %2d    SNR = %5.2f',f,n,2^m,pt,po,snr(4)),'FontSize',fsiz)
xlabel('Frequenz / Abtastfrequenz','FontSize',fsiz)
ylabel('Spektrum in dB','FontSize',fsiz)
line([0 0.5],sm(4)*[1 1],'LineStyle',':','Color','r')
text(0.505,sm(4),sprintf('%3.2f dB',sm(4)),'FontSize',fsiz-2,'Color','r')
set(lh31(1),'UserData',10*fh31+1)
set(lh31(2),'UserData',10*fh31+2)
set(lh31(3),'UserData',10*fh31+3)
set(lh31(4),'UserData',10*fh31+4)
set(fh31,'KeyPressFcn','sichtbarkeit');
set(lh31(2),'visible','off')
fh32 = fullscreen('Ausschnitt Taylor-Entwicklung');
lh32 = plot((0:k2-1)/k,[l2;e02;r0]');   % Spektrum
axis([f-0.01  f+0.01  ug  5])
ax = axis;
lh32(4) = line([f f],[ax(3) 0],'Color','m');
set(gcf,'Position',pos(2,:))
set(gca,'Position',[0.13 0.07 0.775 0.87])
legend('DDS','Fehler','Referenz','ideal')
title('Ausschnitt','FontSize',fsiz+2)
set(lh32(1),'UserData',10*fh32+1)
set(lh32(2),'UserData',10*fh32+2)
set(lh32(3),'UserData',10*fh32+3)
set(lh32(4),'UserData',10*fh32+4)
set(fh32,'KeyPressFcn','sichtbarkeit');
set(lh32(2),'visible','off')
