function x0 = fftfildemo(b,m,u)
% x0 = fftfildemo(b,m,u)
% Es wird gezeigt, dass ein FIR-Filter mit den Koeffizienten b im 
% Zeitbereich dasselbe liefert wie die Multiplikation der FFTs 
% von Signal und von b. Dies ist also eine praktische Demonstration
% des Faltungssatzes. Im Falle der diskreten Fouriertrnsformation
% von endlichen Signalsegmenten mssen sich diese Segmente um
% den Filtergrad berlappen. Diese Demonstration soll vor allem 
% dieses nahtlose Zusammenflicken zeigen.
% Fest eingestellt ist hier der Filtergrad  g = 30 und die 
% FFT-Lnge 128  mit einer berlappung von  128 - g + 1.
% Erzeugt werden zwei sich entsprechend berlappende Segmente.
% Gezeigt werden drei Diagramme:
% (1) das erste ausgeschnittene Segment
% (2) das zweite ausgeschnittene Segment
% (3) die Summe beider Segmente
% In allen Bildern ist der Ausgang des FFT-Filters (blau) und die 
% Differenz der Ergebnisse von FIR-Filter und FFT-Filter angegeben.
% Parameter (alle optional):
% b  Koeffizienten des FIR-Filters
% m  Segmentlnge fr die FFT (z.B. 128)
% u  berlappung der Signalsegmente (Default: length(b))
%
% Bekannter Fehler: fr m > 333  ist die Legende falsch

% Autor: K. v. d. Heide, e-mail:  v.d.heide@on-line.de
%
% Defaults
% ========
if nargin<2  m = 128; end
if nargin<1  b = fir1(30,0.2); end
if nargin<3  u = length(b); end
fsiz = 12;                   % FontSize
%
% Filterberechnung, Signalerzeugung und -verarbeitung
% ===================================================
g = u - 1;                   % Filtergrad
if m < g+2  
   error('Die Anzahl der Filterkoeffizienten muss kleiner als m sein')
end
n = 2*m - g - 1;             % Lnge der Inputsequenz
x = randn(1,n);              % Eingangssignal ist weisses Rauschen
x1= x(1:m);                  % x teilen in zwei Sequenzen der
x2= x(m-g:n);                % Lnge m mit berlappung von g + 1
fb= fft(b,m);                % m-Punkte-FFT der Koeffizienten
y1= real(ifft(fft(x1).*fb)); % Filterung von x1 im Frequenzbereich
y2= real(ifft(fft(x2).*fb)); % Filterung von x2 im Frequenzbereich
y0= filter(b,1,x);           % zum Vergleich: FIR im Zeitbereich
%
% Darstellung:  y0, y1, y2 vergleichen
% ====================================
xu= 0.5;  xo = n + 0.5;
mi= min([y0,y1,y2])-0.1;
mx= max([y0,y1,y2])+0.1;
z = zeros(1,m-g-1);

h = fullscreen('Filter im Frequenzbereich');
subplot(3,1,1); 
plot((1:n),[y1,z],'w');
patch([xu,g+1.5,g+1.5,xu],[mi,mi,mx,mx],'y')
patch([m+0.5,xo,xo,m+0.5],[mi,mi,mx,mx],'k')
set(gca,'NextPlot','add')
n1 = n-m+g+1;
plot((1:n1),y1,(1:n1),y0(1:n1)-y1,'r.');
axis([xu,xo,mi,mx])
legend('','Ausgabe des FFT-Filters','Differenz beider Verfahren',...
       'berlappungsbereich','ausserhalb des Segments');
title('Segment 1','FontSize',fsiz);

subplot(3,1,2); 
plot((1:n),[z,y2],'w');
patch([xu,m-g+0.5,m-g+0.5,xu],[mi,mi,mx,mx],'k')
patch([m-g-0.5,m+0.5,m+0.5,m-g-0.5],[mi,mi,mx,mx],'y')
set(gca,'NextPlot','add')
n2 = m-g;
plot((n2:n),y2,(n2:n),y0(n2:n)-y2,'r.');
axis([xu,xo,mi,mx])
title('Segment 2','FontSize',fsiz);

subplot(3,1,3); 
plot((1:n),[y1,y2(g+2:m)],(1:n),y0-[y1,y2(g+2:m)],'r.');
patch([xu,g+1.5,g+1.5,xu],[mi,mi,mx,mx],'y')
set(gca,'NextPlot','add')
plot((1:n),[y1,y2(g+2:m)],(1:n),y0-[y1,y2(g+2:m)],'r.');
axis([xu,xo,mi,mx])
title('Segmente 1 und 2 aneinandergefgt','FontSize',fsiz);

str='Vergleich: Filterung im Zeitbereich und Filterung im Frequenzbereich mit';
str=sprintf('%s %4.0f-Punkte-FFT bei berlappung um %3.0f Stellen',str,m,g+1);
str=sprintf('%s wegen Filtergrad %3.0f',str,g);
xlabel(str,'FontSize',fsiz)

