function  x0 = filspezfgraph(ofkt,ufkt,sc,ud,od,koef)
%  x0 = filspezfgraph(ofkt,ufkt,koef) 
%  Darstellung der blichen graphischen Filterspezifikation 
%  und Prfung eines evtl. angegebenen Filters auf Einhaltung.
%  ofkt, ufkt :  Strings mit MATLAB-Ausdrcken in der 
%                Frequenzvariablen f (relativ zur Nyquistfrequenz)
%                zur Festlegung der oberen (unteren) Schranke (linear)
%  sc='lin'   :  lineare Skala 0...1.1,  dB-Skala sonst
%  Mit der Angabe koef wird zustzlich ein Filterfrequenzgang gezeichnet
%  und Verletzungen der Spezifikation in rot deutlich hervorgehoben:
%  koef :        Koeffizienten b  bzw. {b;a} fr FIR- bzw. IIR-Filter
%  x0 = 1     :  Wenn koef angegeben und diese die Spezifikation 
%                erfllen, 0 sonst.
%
%  Beispiel:
%  ofkt = '1.1*(f<=0.3)+0.01*(f>0.3).*(f<=0.5)+4*(f>0.5).*(f<=0.8)+1.1*(f>0.8)';
%  ufkt = '0.9*((f<=0.2)+(f>0.8))';
%  b  = remez(16,[0 0.2 0.3 0.5 0.8 1],[1 1 0 0 1 1],[0.1 1 0.2]);
%  ok = filspezfgraph(ofkt,ufkt,0,-60,20);
%  filspezfgraph(ofkt,ufkt,'lin',0.7,1.3,b)

% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de
%
% Defaults
% ========
col = 0.6*[1 1 1];
if nargin<1  ofkt = 'min(1.1,0.2./f)'; end
if nargin<2  ufkt = 'min(1.1,0.2./(f+0.05))-0.1';end
if nargin<3    sc = 0; end
if sc=='lin'
   if nargin<4 ud = 0; end
   if nargin<5 od = 1.2; end
else
   if nargin<4 ud = -100; end
   if nargin<5 od = 10; end
end
%
% Darstellung
% ===========
fh = fullscreen('Filterspezifikation');
if nargin>5  
   filterplot({koef},0,1,sc,ud,od);
end
n  = 801;                 % Anzahl der Punkte im Frequenzbereich
f  = (0:n-1)/(n-1);       % Frequenzbereich 0..1 in 801 Punkten
uf = eval(ufkt);
of = eval(ofkt);
if sc~='lin'
   uf = 20*log10(abs(uf)+eps); 
   of = 20*log10(abs(of)+eps); 
end
patch([f,reverse(f)],[of,od*ones(1,n)],col)
patch([f,reverse(f)],[uf,ud*ones(1,n)],col)
axis([0,1,ud,od])

if nargin<6 x0 = 0; break; end
%
% Prfung
% =======
if iscell(koef)  
   bb = koef{1}; aa = koef{2};
else
   bb = koef;    aa = 1;  
end
yy = max(1E-20,abs(freqz(bb,aa,f*pi)));
if ~strcmp(sc,'lin') y = 20*log10(yy); else y = yy; end
for k = 1:length(y)  if isnan(y(k))  y(k) = 1E-20; end; end
zh = y>of+1.0e-12;
zt = y<uf-1.0e-12;
x0 = ~any(zh|zt);
ah = zh&delay(~zh,1);   ah(1) = zh(1);
eh = zh&delay(~zh,-1);  eh(n) = zh(n);
vonh = find(ah);
bish = find(eh);
for k=1:length(vonh) 
   patch([f(vonh(k)),f(vonh(k):bish(k)),f(bish(k))],...
         [of(vonh(k)),y(vonh(k):bish(k)),of(bish(k))],'r')
end
at = zt&delay(~zt,1);   at(1) = zt(1);
et = zt&delay(~zt,-1);  et(n) = zt(n);
vont = find(at);
bist = find(et);
for k=1:length(vont) 
   patch([f(vont(k)),f(vont(k):bist(k)),f(bist(k))],...
         [uf(vont(k)),y(vont(k):bist(k)),uf(bist(k))],'r')
end
if x0  title('Das Filter erfllt die Spezifikation','FontSize',13)
else   title('Das Filter verletzt die Spezifikation','FontSize',13)
end
   