function  x0 = filspezgraph(pass,stop,pr,sd,koef,figname)
%  x0 = filspezgraph(pass,stop,pr,sd,koef,figname) 
%  Darstellung der blichen graphischen Filterspezifikation
%  mit Durchlassbereichen pass und Sperrbereichen stop sowie
%  - falls angegeben - den Frequenzgang eines Filters mit den
%  Koeffizienten koef.
%  pass = [p1,p2; p3,p4] Frequenzpaare fr Durchlassbereiche
%  stop = [d1,d2; d3,d4] Frequenzpaare fr Sperrbereiche
%  pr   = Welligkeit im Durchlassbereich in dB
%         Ist pr eine Zahl, so ist der Durchlass +-pr
%         Alternativ ist die Angabe eines Bereiches durch einen
%         Vektor mit zwei Komponenten mglich: [untere Grenze, obere Grenze]
%  sd   = Sperrdmpfung im Sperrbereich in dB
%  Mit der Angabe koef wird zustzlich ein Filterfrequenzgang gezeichnet
%  und Verletzungen der Spezifikation in rot deutlich hervorgehoben:
%  koef :         Koeffizienten b  bzw. {b;a} fr FIR- bzw. IIR-Filter
%  koef = 'add' : In diesem Fall wird die Spezifikation zu einer 
%                 vorhandenen Graphik gezeichnet.
%  figname      : Name des Bildes
%  x0 = 1       : Wenn koef angegeben und diese die Spezifikation 
%                 erfllen, 0 sonst.
%
%  Beispiel: n=9; filspezgraph([0,0.1],[0.2,1],[-2,0],-20,ones(1,n)/n)

% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de
%
%
% Konstanten und Defaults
% =======================
colp = 0.6*[1 1 1];
cols = 0.6*[1 1 1];
if nargin<6  figname = 'Filterspezifikation'; end
if nargin<4  sd = -80; end
sd = -abs(sd);
if nargin<3  pr = 0.5; end
if nargin<2  stop = [0 0.15; 0.55 1]; end
if nargin<1  pass = [0.25 0.35]; end
if length(pr)>1
   up = pr(1);
   op = pr(2);
else
   up = -pr/2;
   op = pr/2;
end
ud = 10*round(sd/10)-20;
od = 10*round((op+10)/10);
[np,k] = size(pass);
if k~=2 error('Mehr als zwei Grenzen fr den Durchlassbereich'); end
[ns,k] = size(stop);
if k~=2 error('Mehr als zwei Grenzen fr den Sperrbereich'); end
%
% Darstellung
% ===========
if ischar(figname)
   if nargin>4  
      if ~ischar(koef)
         fh = fullscreen(figname);
         filterplot({koef},0,1,0,ud,od,1,52); 
      elseif strcmpi(koef,'add')
         fh = gcf;
         set(fh,'NextPlot','add')
      end
   else
      fh = fullscreen(figname);
   end
   for k = 1:np  
      patch(pass(k,[1,2,2,1]),[ud,ud,up,up],colp); 
   end
   ss = [1,1,0,0];
   dd = [op,od,od,op];
   for k = 1:ns
      ss = [ss,stop(k,[1,1,2,2])];
      dd = [dd,op,sd,sd,op]; 
   end
   patch(ss,dd,cols);
   axis([0,1,ud,od])
   set(findobj(fh,'type','patch'),'EraseMode','XOR')
end   

if nargin<5 x0 = 0; break; end
if nargin==5 if strcmpi(koef,'add') x0 = 0; break; end; end
%
% Prfung
% =======
sc = 0;                   % lineare Skalierung nicht mglich
n  = 801;                 % Anzahl der Punkte im Frequenzbereich
f  = (0:n-1)/(n-1);       % Frequenzbereich 0..1 in 801 Punkten
of = op*ones(1,n);        % generelle obere Grenze
for k = 1:ns
   of(find(f>=stop(k,1)&f<=stop(k,2))) = sd;  % Sperrbereiche
end
uf = -1000*ones(1,n);     % generelle untere Grenze (dB)
for k = 1:np
   uf(find(f>=pass(k,1)&f<=pass(k,2))) = up;  % Durchlassbereiche
end
if iscell(koef)  
   bb = koef{1}; aa = koef{2};
else
   bb = koef;    aa = 1;  
end
yy = max(1E-20,abs(freqz(bb,aa,f*pi)));
if ~strcmp(sc,'lin') y = 20*log10(yy); else y = yy; end
for k = 1:length(y)  if isnan(y(k))  y(k) = 1E-20; end; end
zh = y>of+1.0e-12;
zt = y<uf-1.0e-12;
x0 = ~any(zh|zt);
%
% Verletzungen der Spezifikation im Diagramm rot markieren
% ========================================================
if ischar(figname)
   ah = zh&delay(~zh,1);   ah(1) = zh(1);
   eh = zh&delay(~zh,-1);  eh(n) = zh(n);
   vonh = find(ah);
   bish = find(eh);
   for k=1:length(vonh) 
      patch([f(vonh(k)),f(vonh(k):bish(k)),f(bish(k))],...
            [of(vonh(k)),y(vonh(k):bish(k)),of(bish(k))],'r')
   end
   at = zt&delay(~zt,1);   at(1) = zt(1);
   et = zt&delay(~zt,-1);  et(n) = zt(n);
   vont = find(at);
   bist = find(et);
   for k=1:length(vont) 
      patch([f(vont(k)),f(vont(k):bist(k)),f(bist(k))],...
            [uf(vont(k)),y(vont(k):bist(k)),uf(bist(k))],'r')
   end
   if x0  title('Das Filter erfllt die Spezifikation','FontSize',13)
   else   title('Das Filter verletzt die Spezifikation','FontSize',13)
   end
end
