function  [ds,dpo,dpu] = filspezgraph2(pass,stop,pr,sd,koef,dez,figname)
%  [ds,dpo,dpu] = filspezgraph2(pass,stop,pr,sd,koef,dez,figname) 
%  Darstellung der blichen graphischen Filterspezifikation
%  mit Durchlassbereichen pass und Sperrbereichen stop sowie
%  - falls angegeben - den Frequenzgang eines Filters mit den
%  Koeffizienten koef.
%  pass = [p1,p2; p3,p4] Frequenzpaare fr Durchlassbereiche
%  stop = [d1,d2; d3,d4] Frequenzpaare fr Sperrbereiche
%  pr   = Welligkeit im Durchlassbereich in dB
%         Ist pr eine Zahl, so ist der Durchlass +-pr/2
%         Alternativ ist die Angabe eines Bereiches durch einen
%         Vektor mit zwei Komponenten mglich: [untere Grenze, obere Grenze]
%  sd   = Sperrdmpfung im Sperrbereich in dB
%         Im Fall sd=0 wird nur der erste Durchlassbereich pass(1)...pass(2) geprft.
%  Mit der Angabe koef wird zustzlich ein Filterfrequenzgang gezeichnet
%  und Verletzungen der Spezifikation in rot deutlich hervorgehoben:
%  koef = Koeffizienten b  bzw. {{b;a}} fr FIR- bzw. IIR-Filter,
%         im Falle einer Filterkaskade Cell Array aller 
%         Koeffizientenvektoren:  {b1; b2; b3; ...}
%         wo  b1 = Koeffizienten von Filter1; b2 = Koeff. ...
%         Anstelle eines Koeffizientenvektors muss im Falle eines
%         IIR-Filters das Cell Array  {bi; ai} treten.
%  koef = 'add' : In diesem Fall wird die Spezifikation zu einer 
%                 vorhandenen Graphik gezeichnet.
%  dez  = Zeilenvektor, der die Dezimationszahlen der Filter enthlt;
%         z.B.  [1, 2, 4].  Default ist [1  1  1 ...]
%         Wird ein Skalar dez angegeben, so gilt dieser fr alle Filter.
%  figname = Name des Bildes.  Kein Bild, wenn nicht String.
%  ds  = 0 die Spezifikation wird im Sperrbereich erfllt,
%          andernfalls ist ds die maximale Verletzung in dB
%  dpo = 0 die Spezifikation wird im Durchlassbereich nicht berschritten,
%          andernfalls ist dpo die maximale berschreitung in dB
%  dpu = 0 die Spezifikation wird im Durchlassbereich nicht unterschritten,
%          andernfalls ist dpu die maximale Unterschreitung in dB
%
%  Beispiel: n=9; filspezgraph2([0,0.1],[0.2,1],[-2,0],-20,ones(1,n)/n)

% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de
%
%
% Konstanten und Defaults
% =======================
if nargin<7  figname = 'Filterspezifikation'; end
if nargin<5  n = 9; koef = ones(1,n)/n; end
[bz,bs] = size(koef);        % bz = Anzahl der seriellen Filter
if nargin<6  dez = ones(1,bz); end
if length(dez)==1  dz = dez*ones(1,bz); else dz = dez; end
if nargin<4  sd = -80; end
sd = -abs(sd);
if nargin<3  pr = 0.5; end
if nargin<2  stop = [0 0.15; 0.55 1]; end
if nargin<1  pass = [0.25 0.35]; end
if length(pr)>1
   up = pr(1);
   op = pr(2);
else
%   up = -pr/2;
%   op = +pr/2;
   up = -pr;
   op = +pr;
end
ud = 10*round(sd/10)-20;
od = 10*round((op+10)/10);
[np,k] = size(pass);
if k~=2 error('Mehr als zwei Grenzen fr den Durchlassbereich'); end
[ns,k] = size(stop);
if k~=2 error('Mehr als zwei Grenzen fr den Sperrbereich'); end
colp = 0.6*[1 1 1];
cols = 0.6*[1 1 1];
%
% Darstellung
% ===========
if ischar(figname)
   if nargin>4  
      if ~ischar(koef)
         fh = fullscreen(figname);
         filterplot(koef,0,1,0,ud,od,dez,52,1); 
      elseif strcmpi(koef,'add')
         fh = gcf;
         set(fh,'NextPlot','add')
      end
   else
      fh = fullscreen(figname);
   end
   for k = 1:np  
      patch(pass(k,[1,2,2,1]),[ud,ud,up,up],colp); 
   end
   ss = [1,1,0,0];
   dd = [op,od,od,op];
   for k = 1:ns
      ss = [ss,stop(k,[1,1,2,2])];
      dd = [dd,op,sd,sd,op]; 
   end
   patch(ss,dd,cols);
   axis([0,1,ud,od])
   set(findobj(fh,'type','patch'),'EraseMode','XOR')
end   

if nargin<5 x0 = 0; break; end
if nargin==5 if strcmpi(koef,'add') x0 = 0; break; end; end

% Prfung
% =======
sc = 0;                   % lineare Skalierung nicht mglich
n  = 4096;                % Anzahl der Punkte im Frequenzbereich
of = op*ones(1,n);        % generelle obere Grenze
uf = -1E10*ones(1,n);     % generelle untere Grenze
pu = uf;                  % untere Grenze im Durchlassbereich
f  = (0:n-1)/(n-1);       % Frequenzbereich 0..1 in n Punkten
if sd==0
   f = pass(1) + (pass(2)-pass(1))*f;
   pu = up*ones(1,n);
else
   for k = 1:np
      pu(find(f>=pass(k,1)&f<=pass(k,2))) = up;  % Durchlassbereiche
   end
   so = +1000*ones(1,n);
   for k = 1:ns
      so(find(f>=stop(k,1)&f<=stop(k,2))) = sd;  % Sperrbereiche
   end
end

y = ones(size(f));
for k = 1:bz              % Frequenzgang berechnen fr alle Filter
   if iscell(koef{k})  
      bb = koef{k}{1}; 
      aa = koef{k}{2};
   else                
      bb = koef{k};    
      aa = 1;  
   end
   y = y.*max(1E-20,abs(freqz(bb,aa,f*dz(k)*pi))); % Filterprodukt
end
for kk = 1:n  if isnan(y(kk))  y(kk) = 1E-20; end; end
y = 20*log10(y);

dpo = max([0,y-of]);
dpu = max([0,pu-y]);
zh  = y>of;
zt  = y<uf;
if sd  
   ds  = max([0,y-so]); 
else 
   ds = []; 
   break   
end

% Verletzungen der Spezifikation im Diagramm rot markieren
% ========================================================
if ischar(figname)
   ah = zh&delay(~zh,1);   ah(1) = zh(1);
   eh = zh&delay(~zh,-1);  eh(n) = zh(n);
   vonh = find(ah);
   bish = find(eh);
   for k=1:length(vonh) 
      patch([f(vonh(k)),f(vonh(k):bish(k)),f(bish(k))],...
            [of(vonh(k)),y(vonh(k):bish(k)),of(bish(k))],'r')
   end
   at = zt&delay(~zt,1);   at(1) = zt(1);
   et = zt&delay(~zt,-1);  et(n) = zt(n);
   vont = find(at);
   bist = find(et);
   for k=1:length(vont) 
      patch([f(vont(k)),f(vont(k):bist(k)),f(bist(k))],...
            [uf(vont(k)),y(vont(k):bist(k)),uf(bist(k))],'r')
   end
   if ~any([ds,dpo,dpu])  title('Das Filter erfllt die Spezifikation','FontSize',13)
   else                   title('Das Filter verletzt die Spezifikation','FontSize',13)
   end
end
