function sc = freqscan(system,p,hd,f1,f2,nf,a)
% sc = freqscan(system,param,hd,f1,f2,nf,a)
% Antwort eines Systems auf Sinussignale im Frequenzbereich f1...f2.
%
% system :  String, der das System als Ausdruck in MATLAB-Syntax beschreibt.
%           Das Inputsignal im Zeitbereich muss als Variable x gefhrt werden. 
%           Der Ausdruck muss das resultierende Ausgabesignal des Systems im 
%           Zeitbereich mit der selben Abtastrate wie das Inputsignal 
%           zurckgeben.
%      p :  Parameter der Systemfunktion als Cell-Array; 
%           diese sind in system in der Form  p{1}, p{2} etc. zu benutzen
%     hd :  Lnge der Einschwingphase in Abtastschritten
% f1, f2 :  Frequenzgrenzen fr die Untersuchung relativ zur Abtastfrequenz
%     nf :  Rauschebene des Kaiserfensters z.B. 150 (dB)
%           Mit nf=0 wird das Rechteckfenster genommen.
%      a :  Auflsung der Graphik. Die Student-Edition erlaubt nur a = 1 (Default)
%           Es werden 2^(a+5) Inputfrequenzen benutzt. Die Auflsung des Spektrums 
%           ist 2^-(a+7) relativ zur Abtastfrequenz.
%     sc :  128*64-Matrix, die fr 64 quidistante Frequenzen zwischen f1 und f2
%           das Spektrum der Ausgabe des Systems in 128 Punkten fr den Bereich 
%           0...Nyquistfrequenz enthlt.
%           Wenn die Ausgabe einer Variablen zugewiesen wird, gibt es keine Graphik.
%
% Dargestellt wird ber der Inputfrequenz (x) das Spektrum des Ausgabesignals (y,z)
% Die Blickrichtung auf das mit der Maus drehbare Bild ist zunchst [-45, 0],
% so dass der normale Amplitudenfrequenzgang des Systems auf einen vertikalen Strich in
% der Mitte des Bildes projiziert ist. Das Kaiserfenster fhrt zu einer Verbreiterung.
% Alle Produkte nichtlinearer Verarbeitung sowie Aliassignale erscheinen rechts und 
% links der Mitte. Durch Drehung lassen sie sich identifizieren.
% Wenn das Bild selektiert ist, kann die Blickrichtung auch mit den 
% Tasten  a , d , f , t eingestellt werden (siehe Hilfefenster mit Taste h).
%
% Beispiel fr ein lineares Filter:
% b = fir1(100,0.2);  % Entwurf eines Tiefpassfilters
% freqscan('filter(p{1},1,x)',{b},100);
%
% Beispiel fr Aliassignale:
% [koef,srate,dly] = mrdesign(0.05,0.03,1,80,150); % Entwurf eines Multiratenfilters
% freqscan('mrfilter(p{1},p{2},x)',{koef;srate},2*dly)
% [koef,srate,dly] = mrdesign(0.05,0.03,1,80,80); % Entwurf eines Multiratenfilters
% freqscan('mrfilter(p{1},p{2},x)',{koef;srate},2*dly)
% [koef,srate,dly] = mrdesign(0.05,0.03,1,80,40); % Entwurf eines Multiratenfilters
% freqscan('mrfilter(p{1},p{2},x)',{koef;srate},2*dly)
% Die drei Bilder dieses Beispiels zeigen im Vergleich die Wirkung der 
% Interpolationsfilter.
%
% Beispiel fr nichtlineare Operation:
% freqscan('x+0.1*x.^2+0.01*x.^3',{});
%
% Beispiele fr Quantisierung:
% freqscan('floor(x*2^p{1})*2^-p{1}',{8});            %  8 BIT
% freqscan('floor(x*2^p{1})*2^-p{1}',{24},0,0,1,160); % 24 Bit
%
% Die Bilder sollten immer auch gedreht werden!

% Autor:  Klaus von der Heide;  e-mail: v.d.heide@on-line.de

% Defaults und Konstanten
% =======================
if nargin<7   a = 1;     end
m = 2^(a+5);                     % Anzahl der Inputfrequenzen
n = 2^(a+7);                     % Anzahl der Outputfrequenzen
if nargin<6  nf = 150;   end
if nargin<5  f2 = 1;     end
if nargin<4  f1 = f2/m;  end
if nargin<3  hd = 0;     end
if nargin<2  param = {}; end
if nargin<1 
   [koef,srate,dly] = mrdesign(0.1,0.03,1,40,120);
   system = 'mrfilter(koef,srate,x)';
   hd = 2*dly;
end
%df   = (f2-f1)/(m-1);
df   = (f2-f1)/m;
n2   = floor(n/2);
fsiz = 13;                      % FontSize
ee   = 10^-12;
if nf==0
   win  = ones(1,n);            % Rechteckfenster
else
   beta = 0.1102*(nf-8.7);
   win  = kaiser(n,beta)';      % Kaiserfenster
end

% Vermessung
% ==========
t = 1:n+hd;
k = 1;
f = f1;
while f<=f2
   x  = sin(pi*f*t);            % Inputsignal
   y  = eval(system);           % Outputsignal
   fy = fft(y(1+hd:n+hd).*win); % FFT
   sy(:,k) = 20*log10(abs(fy(2:n2+1))+ee)'; % Spektrum in dB
   f  = f + df;
   k  = k + 1;
end
%mx = max(max(sy));
mx = (log2(n)-1)*20*log10(2);
sy = sy - mx;

% Graphik
% =======
%fi = f1 + (f2 - f1)*(0:m-1)/(m-1);
fi = f1 + (f2 - f1)*(0:m)/m;
fo = (1:n2)/n2;
h1 = fullscreen('FreqScan');
surf(fi,fo,sy); 
axis square; 
%axis([f1, f2, 1/n2, 1, max(-nf,min(min(sy))), 10])
axis([f1, f2, 1/n2, 1, -150, 10])
title('Frequenzbereich-Antwort fr Sinus-Signale','FontSize',fsiz);
xlabel('Inputfrequenz','FontSize',fsiz);
ylabel('Outputfrequenz','FontSize',fsiz);
set(gca,'View',[-45 0],'XGrid','off','YGrid','off','ZGrid','on');
set(gcf,'KeyPressFcn','fsqview')
rotate3d