function [uu,vv] = homomorph(s,b)
% [uu,vv] = homomorph(s,b)
% Das Signal x = s(1,:) wird logarithmiert und dann mit einem 
% Filter mit den Koeffizienten b und seinem Komplementrfilter 
% gefiltert. 
% uu ist der mit der Exponentialfunktion zurcktransformierte 
% Durchlassanteil und vv der entsprechende Komplementranteil.
% Anstelle von b kann auch die Grenzfrequenz 0<b<1 fr einen 
% Tiefpass angegeben werden, der dann mit REMEZ entworfen wird.
% Fr Demonstrationszwecke kann anstelle eines Zeilenvektors s 
% auch eine 3*n-Matrix angegeben werden, deren Zeilen den 
% Vektor x und die beiden Komponenten u und v enthalten. 
% Werden nur x und u angegeben, so wird v berechnet nach v = x./u
% Wenn die korrekten Komponenten u und v bekannt sind, so gibt 
% es eine zustzliche Graphik.

% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<2  b = 0.04; end
if length(b)==1
    ds = 150;                             % Dmpfung in dB
    [b,srate,dly] = mrdesign(b,b/2,20*log10(1+1/(10^(ds/20))),ds,ds);
    g = 2*dly;                            % quivalenter Filtergrad
else
   g = length(b)-1;                       % Filtergrad
end
if nargin<1
   n = 16384;                             % Anzahl der Abtastungen
   m = 3;                                 % auch u und v gegeben
   t = 0:n-1;                             % Zeitbereich
   u = (1 + 0.8*sin(2*pi*0.002*t));       % Amplitudenschwankung
   v = sin(2*pi*0.05*t);                  % Trgerschwingung
   x = u.*v;                              % Signal
else 
   [m,n] = size(s);
   switch m
      case 1,  x = s;
      case 2,  x = s(1,:);  u = s(2,:);  v = x./u;
      case 3,  x = s(1,:);  u = s(2,:);  v = s(3,:);
      otherwise  error('homomorph :  die Signalmatrix hat falsche Dimension')
   end   
end
n2   = floor(n/2);
fsiz = 13;                                % FontSize
rr   = 150;                               % Rauschebene in dB fr Kaiserfenster

% homomorphe Transformation
% =========================
e  = exp(1);
mn = min(x);
mx = max(x);
xx = (x - mn)/(mx - mn)*(e - 1/e) + 1/e;  % Skalierung auf Bereich  1/e...e
lx = log(xx);                             % lx im Bereich  -1...+1

% Filter
% ======
luu = mrfilter(b,srate,lx);               % Multiratenfilter
lvv = delay(lx,g/2) - luu;                % Komplementrfilter

% Rcktransformation
% ==================
uu = exp(luu);
vv = exp(lvv);
uu  = (uu - 1/e)/(e - 1/e)*(mx - mn) + mn;
vv  = (vv - 1/e)/(e - 1/e)*(mx - mn) + mn;
%w  = delay(x,g/2)./(u+0.0375);
ww  = delay(x,g/2)./uu;

if nargout>0  break; end

% Graphik
% =======
beta = 0.1102*(rr-8.7);
win  = kaiser(n,beta)';                       % Kaiserfenster
 sx  = fft(x.*win);      sx = abs(sx(1:n2));
suu  = fft(uu.*win);    suu = abs(suu(1:n2));
svv  = fft(vv.*win);    svv = abs(svv(1:n2));
slx  = fft(lx.*win);    slx = abs(slx(1:n2));
sluu = fft(luu.*win);  sluu = abs(sluu(1:n2));
slvv = fft(lvv.*win);  slvv = abs(slvv(1:n2));

fh1 = fullscreen('Filter');
filterplot(b,0,1,0,-150,10,cumprod([1 1./srate]),24,1)

if m>1
   us  = (u - mn)/(mx - mn)*(e - 1/e) + 1/e;  % Skalierung auf Bereich  1/e...e
   vs  = (v - mn)/(mx - mn)*(e - 1/e) + 1/e;  % Skalierung auf Bereich  1/e...e
   lu  = log(us);                             % lu im Bereich  -1...+1
   lv  = log(vs);                             % lv im Bereich  -1...+1
   su  = fft(u.*win);      su = abs(su(1:n2));
   sv  = fft(v.*win);      sv = abs(sv(1:n2));
   slu = fft(lu.*win);    slu = abs(slu(1:n2));
   slv = fft(lv.*win);    slv = abs(slv(1:n2));
   fh2 = fullscreen('Teilsignale');
   plot([delay(lx,g/2);delay(lu,g/2);delay(lv,g/2)]')
   legend('log(x)','log(u)','log(v)')
   axis([g min(length(x),g+1000) -1.2  +1.2])
   title('korrekte Teilsignale im transformierten Bereich :   log(x) = log(u) + log(v)','FontSize',fsiz)
   xlabel('Zeitbereich','FontSize',fsiz)
   set(fh2,'UserData',{zeros(1,8); 1.2*ones(1,8); {'b';'g';'m';'c'}});
   set(fh2,'KeyPressFcn','hervorhebung')
end

fh3 = fullscreen('Spektren');
a1 = axes('Position',[0.05 0.55 0.9 0.4]);
if m>1
   plot((0:n2-1)/n2,[su;sv;sx;suu;svv]')
   legend('u','v','x','u"','v"')
   set(a1,'XLim',[0.001  1],'XScale','log','YScale','log')  
   title('Spektren der Signale x, u, v','FontSize',fsiz)
   a2 = axes('Position',[0.05 0.05 0.9 0.4]);
   plot((0:n2-1)/n2,[slu;slv;slx;sluu;slvv]')
   set(a2,'XLim',[0.001  1],'XScale','log','YScale','log')  
   legend('log(u)','log(v)','log(x)','log(u'' )','log(v'' )')
   title('Spektren von log(x), log(u), log(v)','FontSize',fsiz)
else
   plot((0:n2-1)/n2,[sx;suu;svv]')
   legend('x','u''','v''')
   set(a1,'XLim',[0.001  1],'XScale','log','YScale','log')  
   title('Spektren der Signale x, u'', v''','FontSize',fsiz)
   a2 = axes('Position',[0.05 0.05 0.9 0.4]);
   plot((0:n2-1)/n2,[slx;sluu;slvv]')
   set(a2,'XLim',[0.001  1],'XScale','log','YScale','log')  
   legend('log(x)','log(u'' )','log(v'' )')
   title('Spektren von log(x), log(u), log(v)','FontSize',fsiz)
end
set(fh3,'UserData',{zeros(1,8); 1.2*ones(1,8); {'b';'g';'m';'c'}});
set(fh3,'KeyPressFcn','hervorhebungp')

fh4 = fullscreen('Rcktransformation');
plot([delay(x,g/2);uu;vv;ww]')
legend('x','u''','v''','w''')
axis([g min(length(x),g+1000) -10 +10])
xlabel('Zeitbereich','FontSize',fsiz)
set(fh4,'UserData',{zeros(1,8); 1.2*ones(1,8); {'b';'g';'m';'c'}});
set(fh4,'KeyPressFcn','hervorhebung')

% Audioausgabe
% ============
soundsc(x,8000);  pause(3)
soundsc(uu,8000); pause(3)
soundsc(vv,8000);


