function [b2,a2,g] = lcascade(b,a,c)
% [b2,a2,g] = lcascade(b,a,c)
% Kaskadierung eines durch seine bertragungsfunktion [b,a] gegebenen 
% stabilen Systems in eine Kaskadenstruktur aus Systemen zweiter Ordnung. 
% Die Zeilen von b2 und a2 beinhalten die Koeffizienten des Nenners
% bzw. des Zhlers der zugehrigen Systeme zweiter Ordnung.
% Die Verstrkung g wird wie folgt auf die Koeffizienten b verteilt:
%  c = 0      :  gar nicht
%  c = 1      :  auf das erste System zweiter Ordnung
%  c = 2      :  auf alle Systeme gleichermassen
%  c<0 | c>1  :  gleiche Maximalverstrkung in allen Teilsystemen (Default)
%
% Falls das Resultat nicht auf Variable zugewiesen wird, gibt es
% graphische Darstellungen fr die Frequenzgnge der Teilsysteme 
% und des Gesamtsystems.
%
% Methode:  1. tf2zp;  2. zp2sos  (siehe dort)

% Defaults
% ========
if nargin<3  c = 3; end
if nargin<1  [b,a] = ellip(6,1,80,0.1); end
%
% Transformation
% ==============
[z,p,v] = tf2zp(b,a);
[sos,h] = zp2sos(z,p,v);
a2 = sos(:,4:6);
b2 = sos(:,1:3);
%
% Anpassung der Verstrkung
% =========================
g  = 1.0;
switch  c
   case 0, g = h;
   case 1,   b2(1,:) = v*b2(1,:);
   case 2,   [n,m] = size(b2); b2 = v^(1/n) * b2;
   otherwise
      [n,m] = size(b2);
      for k = 1:n
         y(k) = max(abs(freqz(b2(k,:),a2(k,:))));
      end
      r  = (v*prod(y))^(1/n);
      for k = 1:n
         b2(k,:) = r/y(k) * b2(k,:);
      end
end
%
% Diagramme
% =========
if nargout<1
   if nargin<1
      q = 1;
      fh0 = fullscreen('ursprngliches Filter','w',1,[3 1; 1 1]);
      filterplot({{b;a}},0,1,0,-200,100,1,32,1)
      title('Frequenzgang des ursprnglichen Filters','FontSize',13)
   else
      q = 0;
   end
   [n,m] = size(b2);
   s = {{b2(1,:);a2(1,:)}};
   for k = 2:n  
      s = cat(1,s,{{b2(k,:);a2(k,:)}}); 
   end
   fh1 = fullscreen('Kaskade','w',1,[2+q 1; 1+q 1]);
   filterplot(s,0,1,0,-200,100,1,32,1)
   title('Gesamtfrequenzgang der Kaskade','FontSize',13)
   fh2 = fullscreen('Teilsysteme','w',1,[2+q 1; 2+q 1]);
   filterplot(s,0,1,0,-200,100,1,32,0)
   title('Frequenzgnge der Teilsysteme','FontSize',13)
end
   