function [koef,srate,dly,aw] = mrbpdesign(bw,tw,pr,sd,sdi,dec)
% [koef,srate,dly,aw] = mrbpdesign(bw,tw,pr,sd,sdi,dec)
% Entwurf eines Multiraten-Bandpassfilters mit folgenden Eigenschaften:
%              Die Mitte des Durchlassbereiches ist Abtastfrequenz/4
% bw         : Breite des Durchlassgebietes relativ zur Nyquistfrequenz
% tw         : Breite des bergangsbereiches
% pr         : Welligkeit im Durchlassbereich (dB)
% sd         : Dmpfung im Sperrbereich (dB)
% sdi        : evtl. von sd abweichende Dmpfung der Interpolationsfilter.
% sdi = 0    : Das Multiratenfilter soll mit der tiefsten Abtastrate
%              ausgeben. Eine Interpolation findet also nicht statt.
% sdi ~= 0   : Das Multiratenfilter soll zurckinterpolieren auf die 
%              Abtastrate des Eingangs. Dies erfolgt unter Verwendung 
%              entsprechender Interpolationsfilter.
% dec        : mit diesem Wert kann das Verhltnis der Inputabtastrate 
%              zur Abtastrate des Outputs vorgegeben werden. Es muss 
%              allerdings ein Produkt ungerader Zahlen sein. 
%              dec = 25 bewirkt z.B. zwei Dezimationen um 1/5 auch 
%              dann, wenn drei um 1/3 mglich wren. Die Dezimation 
%              wird auch dann, wenn weitere Dezimation mglich wre, 
%              nur bis dez durchgefhrt.
%              Alternativ kann ein Vektor angegeben werden, der 
%              zulssige Dezimationsfaktoren enthlt, z.B. [5 7].
%              Mchte man nur einen Dezimationsfaktor zulassen, so 
%              gibt man diesen doppelt an:  [3 3] .  Im Gegensatz 
%              zu  dez = 9  wird in diesem Fall so oft wie mglich 
%              um 3 dezimiert.
%
% Ausgabe:
% koef  = {b1; b2; ... bn};    b1 = Koeffizientenvektor des ersten Filters etc.
% srate = [sr1, sr2, ... srn]; Abtastratenfaktoren der einzelnen Filter,
%              z.B. 1/3 fr ein Dezimationsfilter,
%              3 fr ein Interpolationsfilter, 1 fr ein Kernfilter.
% dly        : Verzgerung des Filters (ohne bzw. mit Interpolation)
% aw         : Rechenaufwand in Anzahl ntiger Multiply-Accumulate
%              pro Inputabtastung. Dabei ist bercksichtigt, dass alle 
%              Dezimations- und Interpolationsfilter Halbbandfilter sind.
% Wird die Ausgabe nicht einer Variablen zugewiesen, so gibt es einige 
% graphische Darstellungen fr die Frequenzgnge der einzelnen Filter
% und der ganzen Kaskade. Auch wird ein Diagramm fr die Filterspezifikation 
% ausgegeben. Die berschrift "Das Filter verletzt (erfllt) die Spezifikation" 
% ist wegen der mangelnden Auflsung bei schmalen Filtern zu ignorieren. 
% Der Entwurf ist immer als Prototyp aufzufassen, der auf Einhaltung der 
% Spezifikation noch zu prfen und gegebenenfalls manuell zu verbessern ist. 
% Fast immer wird die Spezifikation bertroffen. Die Welligkeit der einzelnen 
% Filter im Durchlassbereich kann sich jedoch unglcklich addieren. Dann ist 
% ein weiterer Versuch mit kleinerer Angabe fr pr angebracht.
%
% koef und srate sind direkt in der Funktion mrfilter verwendbar.
%
% Beispiel: 
% mrbpdesign(0.01,0.002,2,80,0)

% Autor:  Klaus von der Heide; e-mail:  v.d.heide@on-line.de

% Defaults und Konstanten
% =======================
if nargin<5   sdi = 0;      end
if nargin<4    sd = 60;     end
sd = -abs(sd);
sdi= -abs(sdi);
if sdi==0  intpol = 0; else intpol = 1; end
if nargin<3    pr = 1;      end
if nargin<1    bw = 0.08;   end
if nargin<2    tw = 0.2*bw; end
alpha = 1.4;
fsiz  = 13;                                            % FontSize
prt   = pr;
if intpol  prt = pr/2; end
prl   = 10^(prt/20) - 1.0;
sdl   = 10^(sd/20);
%
% Filterstruktur
% ==============
if nargin<6
   mdec  = 7;                                          % grsster Dezimationsschritt
   dc = floor(1/2/(bw/2 + tw));
   decfac = factor(dc);
   while any(ismember(decfac,2)) | any(decfac>mdec) 
      dc = dc - 1; 
      decfac = factor(dc);
   end
else
   if length(dec)==1
      decfac = factor(dec);
   else
      dc = floor(1/2/(bw/2 + tw));
      decfac = factor(dc);
      while any(~ismember(decfac,dec)) 
         dc = dc - 1; 
         decfac = factor(dc);
      end
   end
end
decfac = reverse(decfac);
m = length(decfac);
%   
% Dezimationsfilter
% =================
sr(1) = 1;
r(1)  = bw;
s(1)  = bw*alpha;
for k=1:m                                  % Dezimationsschleife
   r(k+1)  = r(k)*decfac(k);
   s(k+1)  = s(k)*decfac(k);
   g(k)    = 2*(decfac(k)-1);   
   sr(k+1) = 1/decfac(k);
   srate(k)= 1/decfac(k);
   b{k}    = decbandpass(g(k),decfac(k),r(k),0,alpha);
   pass{k} = (1 + r(k)*[-1 +1])/2;
   stop{k} = ((1:2:decfac(k)-2)/decfac(k)/2)'*[1 1]...
      + ones((decfac(k)-1)/2,1)*s(k)/2*[-1 +1];
   [lstk,dummy] = size(stop{k});
   for i=1:lstk
      stop{k} = [stop{k}; 1-stop{k}(lstk-i+1,2), 1-stop{k}(lstk-i+1,1)];
   end
   while ~filspezgraph(pass{k},stop{k},1000,sd,b{k},0)
      b{k} = decbandpass(g(k),decfac(k),r(k),0,alpha);  % Entwurf mit grsserem Grad
      g(k) = g(k) + 2*(decfac(k)-1);
      if g(k)>100  break; end
   end
   if ~filspezgraph(pass{k},stop{k},prt,sd,b{k},0)      % falls Spezifikation im 
      ff = flach(stop{k}(1:(decfac(k)-1)/2,:));         % Durchlass verletzt, dann REMEZ
      ff = ff(2:end);                                   % leider fr REMEZORD ntig
      ff = [ff, pass{k}, 1-reverse(ff)]; 
      aa = [zeros(1,(decfac(k)-1)/2), 1, zeros(1,(decfac(k)-1)/2)];
      dd = [sdl*ones(1,(decfac(k)-1)/2), prl, sdl*ones(1,(decfac(k)-1)/2)];
      rr  = remezord(ff,aa,dd,2,'cell');                % Schtzung der REMEZ-Parameter
      rr{1} = 4*ceil(rr{1}/10);
      if rr{1}/2~=floor(rr{1}/2) rr{1} = rr{1} + 1; end
      rr{2}= max(0.01,min(0.99,rr{2}));
      if any(diff(rr{2})<0)  error('bergangsbereich zu schmal'); end
      b{k} = remez(rr{:});                              % erster Entwurf
      g(k) = rr{1};
      while ~filspezgraph(pass{k},stop{k},prt,sd,b{k},0)
         g(k) = g(k) + 4;
         if g(k)>256  error(sprintf('bei Filtergrad g = 256 fr das Dezimationsfilter%2d aufgegeben',k)); end
         b{k} = remez(g(k),rr{2},rr{3},rr{4});          % Entwurf mit hherem Filtergrad
      end 
   end
   if mod(g(k),4)==2  
      b{k} = b{k}(2:end-1); 
      g(k) = g(k) - 2;
   end
   a(k) = g(k)/2 + 2;
   b{k} = (b{k} + reverse(b{k}))/2;
end
%
% Kernfilter
% ==========
k    = k + 1;
r(k+1)  = r(k);
s(k+1)  = s(k);
rk   = (1 - r(k))/2;
ss   = rk - tw*r(k)/bw;
if ss<0.1  ss = rk/2; end
rr   = remezord([ss  rk  1-rk  1-ss],[0,1,0],[sdl prl sdl],2,'cell');   % Schtzung der REMEZ-Parameter
rr{1}= max(4,rr{1});
if rr{1}/2~=floor(rr{1}/2) rr{1} = rr{1} + 1; end
b{k} = remez(rr{:});                                   % erster Entwurf
g(k) = rr{1};
srate(k)= 1;
pass{k} = [rk  1-rk];
stop{k} = [0  ss; 1-ss  1];
while ~filspezgraph(pass{k},stop{k},prt,sd,b{k},0)
   g(k) = g(k) + 4;
   if g(k)>1024  error(sprintf('bei Filtergrad g = 1024 fr das Kernfilter%2d aufgegeben',k)); end
   b{k} = remez(g(k),rr{2},rr{3},rr{4});               % Entwurf mit hherem Filtergrad
end
a(k) = g(k)/2 + 2;
b{k} = (b{k} + reverse(b{k}))/2;
%
% Interpolationsfilter
% ====================
m = length(g);
if intpol
   for k=1:m-1
      b{k+m}  = b{m-k};                                % Interpolationsfilter identisch 
      sr(k+m) = 1/sr(m-k+1);                           % zu Dezimationsfiltern   
      g(k+m)  = g(m-k);
      a(k+m)  = a(m-k);
      srate(k+m) = 1/srate(m-k);
   end
   if abs(sd-sdi)>2
      for k=(m+1):2*m-1;
         dcf     = decfac(2*m-k);
         r(k)    = r(2*m-k);
         s(k)    = s(2*m-k);
         g(k)    = 2*(dcf-1);   
         b{k}    = decbandpass(g(k),dcf,r(k),0,alpha);
         pass{k} = pass{2*m-k};
         stop{k} = stop{2*m-k};
         while ~filspezgraph(pass{k},stop{k},1000,sdi,b{k},0)
            b{k} = decbandpass(g(k),dcf,r(k),0,alpha);  % Entwurf mit grsserem Grad
            g(k) = g(k) + 2*(dcf-1);
         end
         if ~filspezgraph(pass{k},stop{k},prt,sdi,b{k},0)      % falls Spezifikation im 
            ff = flach(stop{k}(1:(dcf-1)/2,:)); 
            ff = ff(2:end);                                   % leider fr REMEZORD ntig
            ff = [ff, pass{k}, 1-reverse(ff)]; 
            aa = [zeros(1,(dcf-1)/2), 1, zeros(1,(dcf-1)/2)];
            dd = [sdl*ones(1,(dcf-1)/2), prl, sdl*ones(1,(dcf-1)/2)];
            rr  = remezord(ff,aa,dd,2,'cell');                % Schtzung der REMEZ-Parameter
            rr{1} = 4*ceil(rr{1}/10);
            if rr{1}/2~=floor(rr{1}/2) rr{1} = rr{1} + 1; end
            rr{2}= max(0.01,min(0.99,rr{2}));
            b{k} = remez(rr{:});                              % erster Entwurf
            g(k) = rr{1};
            while ~filspezgraph(pass{k},stop{k},prt,sdi,b{k},0)
               g(k) = g(k) + 4;
               if g(k)>256  error(sprintf('bei Filtergrad g = 256 fr das Interpolationsfilter%2d aufgegeben',k)); end
               b{k} = remez(g(k),rr{2},rr{3},rr{4});          % Entwurf mit hherem Filtergrad
            end 
         end
         if mod(g(k),4)==2  
            b{k} = b{k}(2:end-1); 
            g(k) = g(k) - 2;
         end
         a(k) = g(k)/2 + 2;
         b{k} = (b{k} + reverse(b{k}))/2;
      end
   end
end

%
% Ausgabe
% =======
koef = b';
dez  = cumprod(1./sr);
fpd  = filterplot(koef,0.5-0.5*bw,0.5+0.5*bw,0,-3*pr,+3*pr,dez,52,1);
fpd  = fpd(2,:);
b{k} = b{k}*10^((min(fpd)+max(fpd))/40);
koef = b';
dly  = dez*g'/2;
aw   = cumprod(srate)*a';
%
% Darstellung des Frequenzganges
% ==============================
if nargout<1
   for k=1:length(dez)
      if srate(k)>1.5&abs(sd-sdi)<3  break; end
      if srate(k)<1,
         filspezgraph(pass{k},stop{k},pr,sd,b{k},sprintf('Filter%2d',k));
         title(sprintf('Dezimationsfilter%2d :  Halbbandfilter vom Grad %3d',k,g(k)),'FontSize',fsiz)     
      elseif srate(k)==1, 
            filspezgraph(pass{k},stop{k},pr,sd,b{k},sprintf('Filter%2d',k));
            title(sprintf('Kernfilter :  Halbbandfilter vom Grad %3d',g(k)),'FontSize',fsiz)
      else
            filspezgraph(pass{k},stop{k},pr,sdi,b{k},sprintf('Filter%2d',k));
            title(sprintf('Interpolationsfilter%2d :  Halbbandfilter vom Grad %3d',k,g(k)),'FontSize',fsiz)     
      end
      n = k;
   end
   hl1 = fullscreen('Gesamtfrequenzgang');
   filterplot(koef(1:n),0,1,0,sd-20,10,dez(1:n),52,1)
   title('Gesamtfrequenzgang ohne Interpolation','FontSize',fsiz)
   hl2 = fullscreen('Einzelrequenzgnge');
   filterplot(koef(1:n),0,1,0,sd-20,10,dez(1:n),52,0)
   title('Frequenzgnge aller Filter','FontSize',fsiz)
   hl3 = fullscreen('Durchlass');
   filterplot(koef(1:n),0.5-0.6*bw,0.5+0.6*bw,0,-3*pr,+3*pr,dez(1:n),52,1)
   if intpol
      title(sprintf('Gesamtfrequenzgang mit Interpolation,   Rechenaufwand =%8.2f,   Verzgerung =%6d',aw,dly),'FontSize',fsiz)
   else
      title(sprintf('Gesamtfrequenzgang,   Rechenaufwand =%8.2f,   Verzgerung =%6d',aw,dly),'FontSize',fsiz)
   end
   pss = 0.5 + [-1 +1]*bw/2;
   stp = [0  0.5-(bw/2+tw); 0.5+(bw/2+tw)  1]; 
%   [ds,dpo,dpu] = filspezgraph2(pss,stp,pr,sd,koef,dez,0);
%   if dpo&~dpu  koef{1} = koef{1}/(10^(dpo/20)), end
%   if ~dpo&dpu  koef{1} = koef{1}/(10^(dpu/20)), end
   [ds,dpo,dpu] = filspezgraph2(pss,stp,pr,sd,koef(1:n),dez(1:n),'Filterspezifikation');
end
