function [koef,dly,aw] = mrdesign(bw,tw,pr,sd,sdi)
% [koef,dly,aw] = mrdesign(bw,tw,pr,sd,sdi)
% Entwurf eines Multiraten-Tiefpassfilters mit folgenden Eigenschaften:
% bw         : Breite des Durchlassgebietes relativ zur Nyquistfrequenz
% tw         : Breite des bergangsbereiches
% pr         : Welligkeit im Durchlassbereich (dB)
% sd         : Dmpfung im Sperrbereich (dB)
% sdi        : evtl. von sd abweichende Dmpfung der Interpolationsfilter.
% sdi = 0    : Das Multiratenfilter soll mit der tiefsten Abtastrate
%              ausgeben. Eine Interpolation findet also nicht statt.
% sdi ~= 0   : Das Multiratenfilter soll zurckinterpolieren auf die 
%              Abtastrate des Eingangs. Dies erfolgt unter Verwendung 
%              entsprechender Interpolationsfilter.
%
% Ausgabe:
% koef.c = {b1; b2; ... bn};    b1 = Koeffizientenvektor des ersten Filters etc.
% koef.r = [sr1, sr2, ... srn]; Abtastratenfaktoren der einzelnen Filter,
%              z.B. 1/2 fr ein Dezimationsfilter,
%              2 fr ein Interpolationsfilter, 1 fr ein Kernfilter.
% dly        : Verzgerung des Filters (ohne bzw. mit Interpolation)
% aw         : Rechenaufwand in Anzahl ntiger Multiply-Accumulate
%              pro Inputabtastung. Dabei ist bercksichtigt, dass alle 
%              Dezimations- und Interpolationsfilter Halbbandfilter sind.
% Wird die Ausgabe nicht einer Variablen zugewiesen, so gibt es eine 
% graphische Darstellung des Frequenzganges.
% koef und srate sind direkt in der Funktion mrfilter verwendbar.
%
% Der Entwurf ist ein Prototyp, der auf Einhaltung der Spezifikation noch 
% zu prfen und gegebenenfalls manuell zu verbessern ist. Fast immer wird 
% jedoch die Spezifikation bertroffen.
%
% Beispiel: 
% koef = mrdesign2(0.005,0.002,1,60,60);
% t = (1:8000)/2;
% x = 0.5*sin(2*pi*0.005*t) + randn(size(t));
% mrfilter2(koef,x);

% Autor:  Klaus von der Heide; e-mail:  v.d.heide@on-line.de

% Defaults und Konstanten
% =======================
if nargin<5  sdi = 0;           end
if nargin<4   sd = 60;          end
sd = -abs(sd);
sdi= -abs(sdi);
if sdi==0   intpol = 0; else intpol = 1; end
if nargin<3     pr = 1;         end
if nargin<1     bw = 0.08;      end
if nargin<2     tw = 0.3*bw;    end
alpha = 1.01 + min(tw/bw,0.5*(tw/bw)^1.3);
fsiz  = 13;                                            % FontSize
prt   = pr;
if intpol  prt = pr/2; end
prl   = 10^(prt/20) - 1.0;
sdl   = 10^(sd/20);

% Filterstruktur
% ==============
k  = 0;
sr(1) = 1;
while (bw+tw)*2^k<0.4                                  % Dezimationsschleife
   k = k + 1;
   if (bw+tw)*2^k>=0.4 
      r(k) = alpha*(bw+tw/4)*2^(k-1);
   else
      r(k) = alpha*bw*2^(k-1);
   end
   s(k) = 1 - r(k);                                    % Pass/Stop-Symmetrie
   sr(k+1)= 1/2;
   srate(k) = 1/2;
   if r(k)<0.005                                       % Nullstellen vorgeben
      g(k) = 2;
      [b{k},dummy] = z2koef(s(k)*[1; -1]);
      while ~filspezgraph([0 r(k)],[s(k) 1],prt,sd,b{k},0) & g(k)<20
         g(k) = g(k) + 2;
         [b{k},dummy] = z2koef((s(k):2*r(k)/(g(k)+1):1)');% Entwurf mit grsserem Grad
      end
   else   
     g(k) = 2;   
      b{k} = hblowpass(g(k),r(k));
      while ~filspezgraph([0 r(k)],[s(k) 1],prt,sd,b{k},0)
         g(k) = g(k) + 4;
         b{k} = hblowpass(g(k),r(k));                      % Entwurf mit grsserem Grad
      end
   end
   a(k) = g(k)/2 + 2;
end
f1 = 0;
f2 = 1.2*bw;

% Kernfilter
% ==========
k    = k + 1;
kk   = k;
r(k) = bw*2^(k-1);
s(k) = (bw+tw)*2^(k-1);
rr   = remezord([r(k),s(k)],[1,0],[prl sdl],2,'cell'); % Schtzung der REMEZ-Parameter
rr{1}= max(4,rr{1});
if rr{1}/2~=floor(rr{1}/2) rr{1} = rr{1} + 1; end
b{k} = remez(rr{:});                                   % erster Entwurf
g(k) = rr{1};
srate(k) = 1;
while ~filspezgraph([0 r(k)],[s(k) 1],prt,sd,b{k},0)
   g(k) = g(k) + 2;
   b{k} = remez(g(k),rr{2},rr{3},rr{4});               % Entwurf mit hherem Filtergrad
end
a(k) = g(k) + 1;

% Interpolationsfilter
% ====================
if intpol
   m = length(g);
   for k=1:m-1
      b{k+m}  = b{m-k}/sum(b{m-k});                    % Interpolationsfilter identisch 
      sr(k+m) = 1/sr(m-k+1);   
      g(k+m)  = g(m-k);
      a(k+m)  = a(m-k);
      srate(k+m) = 1/srate(m-k);
      r(k+m)  = r(m-k);
      s(k+m)  = s(m-k);
   end
   if abs(sd-sdi)>2
      for k=(m+1):2*m-1;
         r(k) = alpha*bw*2^(2*m-k-1);
         s(k) = 1 - r(k);
         if r(k)<0.005
            g(k) = 2;
            [b{k},dummy] = z2koef(s(k)*[1; -1]);
            while ~filspezgraph([0 r(k)],[s(k) 1],prt,sdi,b{k},0) & g(k)<20
               g(k) = g(k) + 2;
               [b{k},dummy] = z2koef((s(k):2*r(k)/(g(k)+1):1)');% Entwurf mit grsserem Grad
            end
         else   
            g(k) = 2;   
            b{k} = hblowpass(g(k),r(k)); % erster Entwurf mit Grad 4
            while ~filspezgraph([0 r(k)],[s(k) 1],prt,sdi,b{k},0)
               g(k) = g(k) + 4;
               b{k} = hblowpass(g(k),r(k)); % Entwurf mit grsserem Grad
            end
         end
         b{k} = b{k}/sum(b{k});
         a(k) = g(k)/2 + 2;
      end
   end
end

% Ausgabe
% =======
dez  = cumprod(1./sr);
fpd  = filterplot(b',0,bw,'lin',-3*pr,+3*pr,dez,52,1);
fpd  = fpd(2,:);
b{kk}  = b{kk}*2/(min(fpd)+max(fpd));
koef.c = b';
koef.r = srate;
dly  = dez*g'/2;
aw   = cumprod(srate(1:kk))*a(1:kk)' + ...
       0.5*prod(srate(1:kk))*cumprod(srate(kk+1:end))*a(kk+1:end)';

% Darstellung des Frequenzganges
% ==============================
if nargout<1
   pss = [0  bw];
   stp = [bw+tw  1]; 
   for k=1:length(dez)
      switch srate(k)
         case 1/2,
            filspezgraph([0 r(k)],[s(k) 1],pr,sd,b{k},sprintf('Filter%2d',k));
            title(sprintf('Dezimationsfilter%2d :  Halbbandfilter vom Grad %3d',k,g(k)),'FontSize',fsiz)     
         case  1, 
            filspezgraph([0 r(k)],[s(k) 1],pr,sd,b{k},sprintf('Filter%2d',k));
            title(sprintf('Kernfilter :  Grad %3d',g(k)),'FontSize',fsiz)
         case  2,
            filspezgraph([0 r(k)],[s(k) 1],pr,sdi,b{k},sprintf('Filter%2d',k));
            title(sprintf('Interpolationsfilter%2d :  Halbbandfilter vom Grad %3d',k,g(k)),'FontSize',fsiz)     
      end
      n = k;
   end
   hl1 = fullscreen('Gesamtfrequenzgang');
   filterplot(koef.c(1:n),0,1,0,sd-20,10,dez(1:n),52,1)
   title('Gesamtfrequenzgang ohne Interpolation','FontSize',fsiz)
   hl2 = fullscreen('Einzelrequenzgnge');
   filterplot(koef.c(1:n),0,1,0,sd-20,10,dez(1:n),52,0)
   title('Frequenzgnge aller Filter','FontSize',fsiz)
   hl3 = fullscreen('Durchlass');
   filterplot(koef.c(1:n),0,1.2*bw,0,-3*pr,+3*pr,dez(1:n),52,1)
   if intpol
      title(sprintf('Gesamtfrequenzgang mit Interpolation,   Rechenaufwand =%8.2f,   Verzgerung =%6d',aw,dly),'FontSize',fsiz)
   else
      title(sprintf('Gesamtfrequenzgang,   Rechenaufwand =%8.2f,   Verzgerung =%6d',aw,dly),'FontSize',fsiz)
   end
   [ds,dpo,dpu] = filspezgraph2(pss,stp,pr,sd,koef.c(1:n),dez(1:n),'Filterspezifikation');
   
else
   
% Gesamtprfung
% =============
   pss = [0  bw];
   stp = [bw+tw  1]; 
   [dum,dpo,dpu] = filspezgraph2(pss,stp,pr,0,koef.c,dez,0);
   ds = filspezgraph2(pss,stp,100,sd,koef.c(1:kk),dez(1:kk),0);
   pbr = max(abs([dpo dpu]));
   if ds>10   error(['mrbpdesign:  Sperrdmpfung ' num2str(ds) 'dB geringer als spezifiziert!']); end 
   if ds>0  warning(['mrbpdesign:  Sperrdmpfung ' num2str(ds) 'dB geringer als spezifiziert!']); end 
   if pbr>1   error(['mrbpdesign:  Passbandripple ' num2str(pbr) 'dB gsser als spezifiziert!']); end 
   if pbr>0 warning(['mrbpdesign:  Passbandripple ' num2str(pbr) 'dB gsser als spezifiziert!']); end 
end
