function [y,dlay] = mrfilter(koef,dez,x,f1,f2,vs,d1,d2)
% [y,dlay] = mrfilter(koef,dez,x,f1,f2,vs,d1,d2)
% Multiratenfilter:
% koef = {k1; k2; ... kn} :
% k1   :   Koeffizienten des ersten Filters  (usw.)
%          k1 = Vektor : FIR-Filter mit Koeffizientenvektor k1
%          k1 = {b;a}  : IIR-Filter mit Koeffizientenvektoren b,a
%          FIR- und IIR_Filter knnen gemischt verwendet werden.
% dez  :   Dezimations- bzw. Interpolationsfaktoren der Filter
%          Beispiel:  dez = [1/3  1  2]
%          Nach Filter1 Reduktion der Abtastrate um den Faktor 1/3.
%          Filter2 ist das Kernfilter ohne nderung der Abtastrate.
%          Vor Filter3 wird die Abtastrate verdoppelt.
% x    :   Eingangssignal
% Die nachfolgenden Parameter sind optional und dienen nur der Darstellung
% des Frequenzganges. Der Frequenzgang bercksichtigt nur den direkten 
% Durchlass, nicht aber verbleibende Alias bei der Interpolation.
% f1,f2:   untere und obere Frequenz fr den Plot (0 <= f1 < f2 <= 1).
%          Diese Frequenzen sind relativ zur Eingangsabtastrate/2
% vs   :   wenn  vs = 'lin' , dann Skalierung linear, sonst in dB (z.B.vs=0)
% d1,d2:   vertikale Plotgrenzen in dB (d1 < d2), z.B. -120,+10
%
% y    :   Ausgangssignal
% dlay :   Verzgerung des Gesamtfilters in Abtastschritten der Eingabe.
%          Diese Angabe bercksichtigt nur das FIR-Filtern, ist also 
%          bei IIR-Filtern nicht brauchbar.
%
% Wenn die Ausgabe nicht auf eine Variable zugewiesen wird, gibt es 
% drei Grafiken:  
% Frequenzgang des Filters ohne und mit Interpolationsfiltern 
% sowie x und y im Zeitbereich.

% Autor:  Klaus von der Heide;  e-mail: v.d.heide@on-line.de

% Defaults
% ========
if nargin<6  vs = 'db'; end
if nargin<8  d2 = 10;   if strcmp(vs,'lin')  d2 = 1; end; end
if nargin<7  d1 = -100; if strcmp(vs,'lin')  d1 = 0; end; end
if nargin<5  f2 = 1; end
if nargin<4  f1 = 0; end
if nargin<1
   t    = 1:10000;
   x    = sin(2*pi*0.001*t) + randn(size(t));
  [koef,dez] = mrdesign(0.003,0.0008,1,60);
end

nstep = length(koef);
if nstep~=length(dez) error('koef und dez mssen die selbe Lnge haben.'); end

% Rechnung
% ========
xtemp = x;
for step = 1:nstep
   if iscell(koef{step})                 % IIR-Filterkoeffizienten  
      b = koef{step}{1};
      a = koef{step}{2};
   else
      b = koef{step};                    % FIR-Filterkoeffizienten
      a = 1;
   end
   ds(step) = (length(b)-1)/2; 
   df = 1;
   if dez(step)>1
      df = round(dez(step));             % Interpolationsfaktor
      if abs(df-dez(step))>eps error('unzulssiger Interpolationsfaktor.'); end
      xtemp = df*[nullein(xtemp,df-1),zeros(1,df-1)]; % Hochtastung
   end
   xtemp = filter(b,a,xtemp);            % Filtern
   if dez(step)<1
      df = round(1/dez(step));           % Dezimationsfaktor
      if abs(df-1/dez(step))>eps error('unzulssiger Dezimationsfaktor.'); end
      xtemp = xtemp(1:df:length(xtemp)); % Dezimation
   end
end
y    = xtemp;

dv  = cumprod(1./dez);
%dv(1:(length(dv)-1)/2) = dv(1:(length(dv)-1)/2)/dv(1);
dv(1:floor((length(dv)-1)/2)) = dv(1:floor((length(dv)-1)/2))/dv(1);
dlay = dv*ds';

if nargout>0 break; end                  % wenn Zuweisung, dann keine Graphik

% Graphik
% =======
kk = 0;
for k = 1:length(dez)
   if dez(k)<=1
      kk      = kk + 1;
      dk{kk}  = koef{k};
      dkdv(kk)= dv(k);
   end
end
dk = dk';
h1 = fullscreen('Durchlass');
filterplot(koef,f1,f2,vs,-3,+3,dv,32,1)
title('Betragsfrequenzgang des Multiratenfilters mit Interpolationsfiltern','FontSize',13)
h2 = fullscreen('Frequenzgang');
filterplot(dk,0,1,0,d1,d2,dkdv,32,1)
title('Betragsfrequenzgang des Multiratenfilters ohne Interpolationsfilter','FontSize',13)

h3 = fullscreen('Zeitbereich');
plot(1:length(x),delay(x,dlay),'b',length(x)*(1:length(y))/length(y),y,'r')
title(sprintf('Filterung mit Multiratenfilter,   Verzgerung =%4d Abtastschritte',dlay),'FontSize',13)
xlabel('Zeitbereich','FontSize',13)
set(gcf,'UserData',{zeros(1,8); 1.2*ones(1,8);{'b';'g';'m';'c'}},...
        'KeyPressFcn','hervorhebung')
