function [s,fr,snr,sm,smf] = newdds(f,z,pt,po,dith,window)
% [s,fr,snr,sm,smf] = newdds(f,z,pt,po,dith,window)
% Simulation einer DDS mit folgenden Parametern:
%  f   :  Frequenz relativ zur Abtastfrequenz
%         Das Signal s wird analytisch berechnet.  
%         Deshalb ist  0 <= f < 1.0  zulssig.
%         (Default: Zufallszahl)
%  z   :  Vektor der Lngen der Sinustabellen.
%         Zur Zeit sind nur maximal 4 Komponenten zugelassen. 
%         Die Komponenten von z mssen teilerfremd sein.
%         (Default [2047, 2048, 2049])
%  pt  :  Anzahl der Datenbits der ROMs im 2-Komplement    (Default 20)
%  po  :  Anzahl der Datenbits der Ausgabe im 2-Komplement (Default 16)
%  dith:  zum Ausgangssignal wird vor der Quantisierung auf po
%         Bits das Dithersignal dith*(rand-0.5)*2^(1-po) addiert
%         abs(dith)<1 verhindert das Dithern. Sinnvoll ist dith = 1
%         (Default 0)
% window: 0 Rechteck;  1 Hanning;  2 Hamming;  3 Kaiser  (Default 3)
%  s   :  4096 Abtastungen des berechneten analytischen Signals
%  fr  :  tatschlich realisierte Frequenz
%  snr :  Signal/Rauschverhlnis in dB
%  sm  :  minimale Unterdrckung von Nebenfrequenzen in dB
%  smf :  Frequenz des Maximums im Nebenfrequenzspektrum
%
% Wird die Ausgabe keiner Variablen zugewiesen, so wird das 
% Spektrum graphisch dargestellt.
%
% Autor: Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
%=========
if nargin<6   window = 0;  end
if nargin<5     dith = 0;  end
if nargin<3       pt = 20; end
if nargin<4       po = pt; end
if nargin<2        z = [2047, 2048, 2049]; end
if nargin<1        f = 0.8*rand; end
%
% Konstanten
% ==========
lz = length(z);                    % Anzahl der Sinustabellen
if lz<2 | lz>4  error('die Anzahl der Sinustabellen muss 2, 3 oder 4 sein!'); end
if screen==1  
   pos = [2 30 525 548; 532 33 267 546];
   fsiz = 10;                      % FontSize
else
   pos = [4 31 694 716; 703 33 319 714];
   fsiz = 13;                      % FontSize
end

df = 1/prod(z);                    % kleinster Frequenzschritt
nf = round(f/df);                  % f in Frequenzschritten
fr = nf*df;                        % realisierte Frequenz
p  = 1/fr;                         % Periode in Anzahl Abtastungen / Schwingung
ks = ceil(fr*4096):floor(fr*16384);% Anzahl der Schwingungen
d  = abs(ks*p - round(ks*p));      % Samplefehler der periodischen Fortsetzung
md = min(d);                       % Minimalfehler
k  = round(p*ks(min(find(d==md))));% Anzahl der Abtastpunkte
t  = 0:k-1;                        % Zeitbereich
nl = 6.02*po + 50;                 % Rauschebene in dB
pb = round(0.01*k);

switch window                      % Berechnung des gewhlten Fensters
   case 0,  win = ones(1,k);
   case 1,  win = hanning(k)';
   case 2,  win = hamming(k)';
   case 3,  win = kaiser(k,0.1102*(nl-8.7))';
end   

for l = 1:lz                       % CosSinTabellen:
   exptab{l} = round(exp(2*pi*i*(0:z(l)-1)/z(l))*2^(pt-1)*(1-2^(1-pt)))*2^(1-pt);
end
%
% Berechnung der Phasenschritte
% =============================
x  = diophant(z);                  % Lsung der diophantischen Gleichung
pd = mod(nf*x,z);                  % Phasenschritte
pd0= pd==0;                        % Nullschritte
%
% Ideale Ausgabe
% ==============
cs = cos(2*pi*fr*t)*(1-2^(1-pt))^(lz-sum(pd0)); % idealer Cosinus
fc = fft(cs.*win);                 % FFT
mfc= max(abs(fc));                 % Normierungsfaktor
ssc= sum(cs.^2);                   % Leistung
r0 = 20*log10(abs(fc/mfc)+eps);    % Referenzspektrum in dB
%
% Exp-Berechnung
% ==============
for l = 1:lz                       % individuelle DDS
   if pd0(l) 
      si(l,:) = ones(size(t));
   else 
      si(l,:) = exptab{l}(mod(pd(l)*t,z(l))+1);
   end
end
s  = prod(si,1);                   % Produkt aller
s  = s + dith*((rand(size(s)) - 0.5)*2^(1-po)...
       + i*(rand(size(s)) - 0.5)*2^(1-po)); % Dithern 
s  = round(s*2^(po-1))*2^(1-po);   % Quantisierung 
s  = min(1-2^(1-po),max(-1,real(s))) ...
     + i*min(1-2^(1-po),max(-1,imag(s))); % Clippen
ds = real(s) - cs;                 % Fehler der Ausgabe

% Spektrum
% ========
f0 = fft(real(s).*win)/mfc;        % Spektrum
l0 = 20*log10(abs(f0)+eps);        % Spektrum in dB
es = fft(real(ds).*win)/mfc;       % Fehlerspektrum
e0 = 20*log10(abs(es)+eps);        % Spektrum in dB

fm = round(fr*k) + 1;              % Index der Spitze im Spektrum
sm = max(e0([1:fm-pb,fm+pb:round(k/2)])); % Maximum des Nebenfrequenzspektrums
smf= (find(e0==sm)-1)/k;           % Frequenz dieses Maximums
smf= smf(1);                       % nur eines ausgeben
%
% SNR
% ===
snr= 10*log10(sum(ds.^2)/ssc);     % SNR
%
% Ausgabe
% =======
if nargout>0  break; end
%
% Darstellung
% ===========
Kontrollausgabe = sprintf('newdds\n#Abtastpunkte:   %5d\nSollfrequenz:       %15.12f\nFrequenzfehler:     %15.12f\nNebenmaximum bei:   %15.12f\nBetrag:           %6.1f dB\nSNR:              %6.1f dB',k,f,fr-f,smf,sm,snr)
ug = max([-250  min([sm  -6*po])-30]);
fh1 = fullscreen('DDS-Spektrum');
lh  = plot((0:k-1)/k,[l0;e0;r0]');   % Spektrum
axis([0 1/2 ug 5])
ax = axis;
lh(4) = line([f f],[ax(3) 0],'Color','m');
set(gcf,'Position',pos(1,:))
set(gca,'Position',[0.08 0.07 0.85 0.87])
legend('DDS','Fehler','Referenz','ideal')
title(sprintf('Spektrum der NewDDS,    f = %7.6f    pt = %2d    po = %2d    SNR = %5.2f',f,pt,po,snr),'FontSize',fsiz+2)
xlabel('Frequenz / Abtastfrequenz','FontSize',fsiz)
ylabel('Spektrum in dB','FontSize',fsiz)
line([0 0.5],sm*[1 1],'LineStyle',':','Color','r')
text(0.505,sm,sprintf('%3.2f dB',sm),'FontSize',fsiz-2,'Color','r')
text(0.505,-5,'ROM-Gren','FontSize',fsiz-2)
for l = 1:lz
   text(0.505,-5*(l+1),sprintf('%6d',z(l)),'FontSize',fsiz-2)
end
set(lh(1),'UserData',10*fh1+1)
set(lh(2),'UserData',10*fh1+2)
set(lh(3),'UserData',10*fh1+3)
set(lh(4),'UserData',10*fh1+4)
set(fh1,'KeyPressFcn','sichtbarkeit');
set(lh(2),'visible','off')
fh2 = fullscreen('Ausschnitt');
lh2 = plot((0:k-1)/k,[l0;e0;r0]');   % Spektrum
axis([f-0.01  f+0.01  ug  5])
ax = axis;
lh2(4) = line([f f],[ax(3) 0],'Color','m');
set(gcf,'Position',pos(2,:))
set(gca,'Position',[0.13 0.07 0.775 0.87])
legend('DDS','Fehler','Referenz','ideal')
title('Ausschnitt','FontSize',fsiz+2)
set(lh2(1),'UserData',10*fh2+1)
set(lh2(2),'UserData',10*fh2+2)
set(lh2(3),'UserData',10*fh2+3)
set(lh2(4),'UserData',10*fh2+4)
set(fh2,'KeyPressFcn','sichtbarkeit');
set(lh2(2),'visible','off')
