function [x0,y0] = numztrans(q,coord,view)
%  [x0,y0] = numztrans(q,coord,view)
%  errechnet die Z-Transformierte von q numerisch und stellt sie
%  als 3D-Plot dar.
%
%  q             : Sample-Vektor, der die zu transformierende 
%                  Funktion darstellt (Lnge 128...1024 sinnvoll).
%  coord = 'crt' : normales kartesisches Gitter (Default)
%  coord = 'cyl' : Zylinderkoordinaten
%  view          : Betrachtungswinkel 
%                  Beispiel: view = [0 90] ergibt einen Pseudocolorplot.
%  
%  x0 = numtrans(q,coord) und
%  x0 = numtrans(q,coord,view) liefern in x0 die 100*100-Matrix der 
%  Absolutwerte der Z-Transformierten und stellen diese zustzlich 
%  als 3D-Bild dar.
%  Im Fall von coord = 'cyl' wird die Z-Transformierte in Polarkoordinaten 
%  ausgegeben mit Schritten von 4 bzw. 0.05. x0 ist also eine 90*40-Matrix.
%
%  [x0,y0] = numtrans(q)
%  [x0,y0] = numtrans(q,coord)
%  [x0,y0] = numtrans(q,coord,view)
%  Darstellung von  abs(Z-Transformierte)  als 3D-Bild und Ausgabe von:
%  x0 = Figurehandle,  y0 = Axeshandle
%
%  Das 3D-Bild ist mit der Maus drehbar.
%
%  Der Einheitskreis hat aus numerischen Grnden den Radius 1.01

% Autor: K. v. d. Heide, e-mail:  v.d.heide@on-line.de
%
% Defaults
% ========
fsz= 14;                                        % FontSize fr Beschriftung
if nargin<3 view = [10,20]; end                 % Default: Blickrichtung
if nargin<2 coord = 'crt'; end                  % Default: Zylinderkoordinaten
if nargin<1
   k = 64;
   t = (1:k)/k;
   q = (sin(2*pi*10*t)+0.5*sin(2*pi*17*t)).*hanning(k)';
end
r0 = 1.02;                                      % Radius des Einheitskreises
%
% Koordinaten
% ===========
m  = length(q);
if strcmp(coord,'cyl')
   n  = 90;                                     % Phipunkte
   m  = 40;                                     % Radiuspunkte
   [x,y] = pol2cart((ones(m,1)*(0:n)/n*2*pi)',ones(n+1,1)*(m:(2*m-1))/m);
   z  = x + i*y;                                % komplexe Ebene
else
   n  = 100;                                    % Kantenlnge des Plots
   zz = 4*(-n/2:(n-1)/2)/n;                     % Bereich fr Real / Imaginrteil
   z  = ones(n,1)*zz + i*zz'*ones(1,n);         % komplexe Ebene
end
w  = (0:511)/256*pi;                            % 0 ... 2 pi in 512 Punkten
zek= r0*(cos(w) + i*sin(w));                    % Einheitskreis
%
% Funktion
% ========
y  = zeros(size(z));
for k = 1:m  y = y + q(k)*z.^-(k-1); end        % Z-Transformation
%
% Ausgabe
% =======
if (nargout~=0) & (nargin<2) x0 = y; break; end
%
% Darstellung
% ===========
if ~strcmp(coord,'cyl') y = y.*(abs(z)>r0);end  % Inneres des Einheitskreises verwerfen
ly = log(abs(y));                               % zur Darstellung lod(abs(...))
my = max(max(ly));                              % Maximum davon fr Farbskala
yek= zeros(size(zek));
for k = 1:m  yek = yek + q(k)*zek.^-(k-1); end  % Z-Transformation auf Einheitskreis
ylk= log(abs(yek));                             % logarithmisch auf Einheitskreis
fh = fullscreen('Z-Trans 3D');
%set(fh,'keypressfcn','togglevisibility')
if strcmp(coord,'cyl')
   zp = [flach(z(1:n,1:m-1)); flach(z(2:n+1,1:m-1));
         flach(z(2:n+1,2:m)); flach(z(1:n,2:m))];
   yp = [flach(ly(1:n,1:m-1));flach(ly(2:n+1,1:m-1));
         flach(ly(2:n+1,2:m));flach(ly(1:n,2:m))];
   patch(real(zp),imag(zp),yp,mean(yp)/my)
else
   surf(zz,zz,ly); axis square; axis([-2,2,-2,2,my-10,my])
end
line(real(zek),imag(zek),ylk,'Color','k','LineWidth',2)
xlabel('real(Z)','FontSize',fsz); 
ylabel('imag(Z)','FontSize',fsz); 
zlabel('log(abs(Z-Transformierte))','FontSize',fsz)
title('Betrag der bertragungsfunktion','FontSize',fsz)
axis square
set(gca,'View',view)
rotate3d on
if nargout==1 x0 = y(1:n,:); else x0 = [fh,gca]; end