function [z,z0,fr] = rec2dds(f,pt,po,window,p0)
% [z,z0,fr] = rec2dds(f,pt,po,window,p0)
% Rekursive Generierung der Kreisfunktion durch 
%    z(k) = 2*b*z(k-1) - z(k-2)
% mit  c = cos(2*pi*round(f*2^pt)*2^-(pt-1))
% und z(1) = cos(p0)
% Die Arithmetik wird mit pt Bits durchgefhrt wird und die 
% Ausgabe mit po Bits erfolgt.
%
% f  :    Frequenz relativ zur Abtastfrequenz     (Default 0.22)
% pt :    Anzahl der Bits zur internen Berechnung (Default 20)
% po :    Anzahl der Bits zur Ausgabe             (Default 16)
% window: 0 Rechteck;  1 Hanning;  2 Hamming;  3 Kaiser  (Default 0)
% p0 :    Phasenwinkel des ersten Wertes
% z  :    rekursiv erzeugte Kreisfunktion, die Lnge dieses Vektors
%         liegt zwischen 1024 und 16384 Abtastpunkten
% z0 :    ideale Kreisfunktion (Lnge wie z)
% fr :    realisierte Frequenz
%
% Autor: Klaus von der Heide, e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<5     p0 = 0;    end
if nargin<4 window = 0;    end
if nargin<2     pt = 20;   end
if nargin<3     po = pt;   end
if nargin<1      f = 0.22; end

% Konstanten
% ==========
if screen==1  
   pos = [2 30 525 548; 532 33 267 546];
   fsiz = 11;                      % FontSize
else          
   pos = [4 31 694 716; 703 33 319 714];
   fsiz = 13;                      % FontSize
end
n = pt;

% Die wirklich realisierte Frequenz ergibt sich aus dem ersten Durchgang
% der Berechnung. Diese bestimmt aber die optimale Lnge der FFT, so dass
% gegebenenfalls ein zweiter Durchgang erforderlich ist. 

fr = f;                            % Schtzung der realisierten Frequenz
for it=1:2                         % zwei Berechnungsdurchgnge
%------------------------- it-Schleife ---------------------------------   
p  = 1/fr;                         % Periode in Anzahl Abtastungen / Schwingung
k  = ceil(fr*4096):floor(fr*16384);% Anzahl der Schwingungen
d  = abs(k*p - round(k*p));        % Samplefehler der periodischen Fortsetzung
md = min(d);                       % Minimalfehler
m  = round(p*k(min(find(d==md)))); % Anzahl der Abtastpunkte
b  = 2^(pt-1);                     % Niveaus im 2-Komplement
b1 = 1/b;                          % Niveau-Abstand im 2-Komplement
mx = 1 - b1;                       % Maximum im 2-Komplement
bo  = 2^(po-1);                    % Niveaus im 2-Komplement
bo1 = 1/bo;                        % Niveau-Abstand im 2-Komplement
mxo = 1 - bo1;                     % Maximum im 2-Komplement
c = cos(2*pi*round(f*2^(n-1))*2^-(n-1));
c  = min([round(c*b)*b1,mx]);
nl = 6.02*po + 50;                 % Rauschebene in dB
pb = round(0.05*m);

switch window
   case 0,  win = ones(1,m);
   case 1,  win = hanning(m)';
   case 2,  win = hamming(m)';
   case 3,  win = kaiser(m,0.1102*(nl-8.7))';
end   

% Ideale Kreisfunktion
% ====================
z0 = cos(2*pi*f*(0:m-1)+p0);       % ideale Kreisfunktion
fc = fft(z0.*win);                 % FFT
mfc= max(abs(fc));                 % Normierungsfaktor
ssc= sum(real(z0).^2);             % Leistung
r0 = 20*log10(abs(fc/mfc)+eps);    % Referenzspektrum in dB

% Rekursive Berechnung
% ====================
z(1) = cos(p0);                    % Anfangswert
z(2) = cos(p0 + 2*pi*round(f*2^(n-1))*2^-(n-1));
z(1) = max(-1,min([mx,round(z(1)*b)*b1]));
z(2) = max(-1,min([mx,round(z(2)*b)*b1]));
%z(1) = 1;  z(2) = c/2;
for k=3:m  
   z(k) = max(-1,min([mx,round((2*z(k-1)*c-z(k-2))*b)*b1]));
end
z  = round(z(1:m)*bo)*bo1;         % Quantisierung
z  = min(mxo,max(-1,real(z)));     % Clippen
ds = z - z0;                       % Fehler

% Spektrum
% ========
ff   = abs(fft(z.*win))/mfc;       % DDS-Spektrum
ffdb = 20*log10(ff+eps);           % in dB
ef   = abs(fft(ds.*win))/mfc;      % Fehlerspektrum
efdb = 20*log10(ef+eps);           % in dB
fr   = (find(max(ffdb(1:floor(m/2)))==ffdb)-1)/m;% realisierte Frequenz
if abs(f-fr)*m < 0.51  break; end  % keine Verbesserung durch Iteration
%------------------------- Ende der it-Schleife -------------------------   
end                                % Ende der Iteration

%
% SNR
% ===
snr= 10*log10(sum(ds.^2)/ssc);     % SNR
ix = round(fr*m) + 1;              % Index der Frequenz
sm = max(efdb([1:ix-pb,ix+pb:floor(m/2)])); % Maximum des Nebenfrequenzspektrums
smf= (find(efdb==sm)-1)/k;         % Frequenz dieses Maximums
smf= smf(1);                       % nur eines ausgeben

% Ausgabe
% =======
Kontrollausgabe = sprintf('rec2dds\n#Abtastpunkte:   %5d\nSollfrequenz:       %15.12f\nFrequenzfehler:     %15.12f\nNebenmaximum bei:   %15.12f\nBetrag:           %6.1f dB\nSNR:              %6.1f dB\nAnfangsphase:       %15.12f\n',m,f,fr-f,smf,sm,snr,p0)
ug = max([-250  min([sm  -6*po])-30]);
fh1 = fullscreen('Rec2DDS');
lh = plot((0:m-1)/m,[ffdb;efdb;r0]');
axis([0  0.5  ug  5])
lh(4) = line([f f],[ug 0],'Color','m');
set(gcf,'Position',pos(1,:))
set(gca,'Position',[0.08 0.07 0.85 0.87])
title(sprintf('Rekursive DDS:   f = %1.6f   %3d-Bit-Arithmetik   Ausgabe: %2d Bit   SNR = %5.2f',f,pt,po,snr),'FontSize',fsiz-1)
xlabel('Frequenz / Abtastfrequenz','FontSize',fsiz)
line([0 1],sm*[1 1],'LineStyle',':','Color','r')
text(1.01,sm,sprintf('%3.2f dB',sm),'FontSize',fsiz-2,'Color','r')
legend('DDS','Fehler','Referenz','ideal')
set(lh(1),'UserData',10*fh1+1)
set(lh(2),'UserData',10*fh1+2)
set(lh(3),'UserData',10*fh1+3)
set(lh(4),'UserData',10*fh1+4)
set(fh1,'KeyPressFcn','sichtbarkeit');
set(lh(2:3),'visible','off')
fh2 = fullscreen('Ausschnitt');
lh2 = plot((0:m-1)/m,[ffdb;efdb;r0]');
axis([f-0.01  f+0.01  ug  5])
ax = axis;
lh2(4) = line([f f],[ax(3) 0],'Color','m');
set(gcf,'Position',pos(2,:))
set(gca,'Position',[0.13 0.07 0.775 0.87])
title(sprintf('Ausschnitt',f,po),'FontSize',fsiz)
legend('DDS','Fehler','Referenz','ideal')
set(lh2(1),'UserData',10*fh2+1)
set(lh2(2),'UserData',10*fh2+2)
set(lh2(3),'UserData',10*fh2+3)
set(lh2(4),'UserData',10*fh2+4)
set(fh2,'KeyPressFcn','sichtbarkeit');
set(lh2(2:3),'visible','off')
