function  y = resample(x,t1,t2,k,r,n)
% y = resample(x,t1,t2,k,r,n)
% Interpolation des diskreten Signals x durch Shannon-Synthese.
% Die sinc-Funktion wird Realisierungs-nahe als Tabelle gespeichert.
%
% x   :  Eingabesignal
% t1  :  Vektor der quidistanten Zeitmarken fr die Abtastung von x
%        x und t1 mssen die selbe Lnge haben.
% t2  :  Vektor der quidistanten Zeitmarken fr die zu berechnende Ausgabe y.
%        Die Elemente von t2 mssen zwischen dem ersten und dem letzten Wert
%        von t1 liegen:  t1(1) <= t2(1) <...< t2(length(t2)) <= t1(length(t1))
% k   :  Die zur Shannon-Synthese verwendete sinc-Funktion wird nur  
%        im Intervall -k ... +k ausgewertet.
% r   :  Die sinc-Funktion wird mit einem Cosinus-Roll-Off-Fenster mit 
%        dem Roll-Off-Faktor r bewertet: 0 <= r <= 1 (r = 0 fr Rechteckfenster)
% n   :  Zahl der sinc-Tabellenwerte zwischen zwei Abtastungen von t1.
%        Die sinc-Tabelle hat damit insgesamt die Lnge  2*k*n + 1.
%
% Zur Berechnung von Ausgabewerten y ausserhalb des Zeitbereichs
% t1(k) ... t1(length(t1)-k  werden nicht vorhandene Eingabewerte von x,
% d.h. solche ausserhalb von t1 bentigt. Diese werden als Null angenommen.
% 
% Wenn die Ausgabe nicht einer Variablen zugewiesen wird, gibt es eine
% Graphik mit den Abtastwerten des Eingangs- und des Ausgangssignals.
%
% Beispiele: 
% [u,v] = ellip(6,0.1,60,0.6);  x = filter(u,v,randn(1,200)); 
% resample(x,1:length(x),100+(1:1/11:50),10,0,8);    % synchrone Hochtastung
% resample(x,1:length(x),100+(1:1/11:50),10,0,1);    % Tabelle zu klein
% resample(x,1:length(x),100+(1.11:pi/7:50),10,0,8); % asynchrone Hochtastung
% resample(x,1:length(x),100+(1.11:pi/3:50),10,0,8); % asynchrone Umtastung
% resample(x,1:length(x),100+(1:1/11:50),1,0,8);     % k zu klein
% umtastung(x,1:length(x),100+(1:1/11:50),b,8);    % falsche Durchlassbereiche

% Autor: K. v. d. Heide, e-mail:  v.d.heide@on-line.de
%
% Konstanten und Defaults
% =======================
if nargin<1
   [u,v] = ellip(6,0.1,60,0.2); 
   x  = filter(u,v,randn(1,100));
end
if nargin<2  t1 = 1:length(x); end
if nargin<3  t2 = t1(1) + rand + (0:1.7*rand+0.3:t1(length(t1))-t1(2)); end
if nargin<4  k = 15; end
if nargin<5  r = 0; end
if nargin<6  n = 8; end

if r>1 | r<0  error('Der CosRollOff-Faktor r muss im Intervall 0 ... 1 liegen'); end

m1 = length(x);
if m1~= length(t1)  
   error('Input x und dessen Zeitmarken mssen dieselbe Lnge haben.');
end
m2 = length(t2);
dt1= t1(2)-t1(1);
x0 = [zeros(1,k+1),x,zeros(1,k)];        % Nullen anfgen

% Tabelle fr die Impulsantwort erstellen
% =======================================
tb = sinc(-k:1/n:+k);
if r>0 tb = tb.*cosrolloff(-1-r:(1+r)/k/n:+1+r,r); end
mt = length(tb);
%
% Umtastung
% =========
for i = 1:m2                               % Schleife ber alle Ausgabeabtastungen
   tt2 = t2(i);                            % fr diesen Zeitpunkt jetzt berechnen
%   tt1 = max(t1.*(t1<=tt2));               % der letzte Abtastpunkt t1(j) vor t2(i)
%   j   = max((1:m1).*(tt1==t1))+k+1;       % Index j des letzten x(j) vor t2(i)
   j   = max(find(t1<=tt2&tt2<=t1+dt1));   % der letzte Abtastpunkt t1(j) vor t2(i)
   if isempty(j)
      y(i) = 0;
   else
      tt1 = t1(j);
      j   = j + k + 1;
      xx  = (tt2 - tt1)/dt1*n;
      x1  = floor(xx);                      % Offset fr Tabelle
      x2  = xx - x1;                        % Offset fr Interpolation in Tabelle
      aa  = tb(x1+1:n:mt-1)*x0(j+k:-1:j-k+1)'; % erster Tabellenwert
      bb  = tb(x1+2:n:mt)*x0(j+k:-1:j-k+1)';   % zweiter Tabellenwert
      y(i)= aa + (bb - aa)*x2;              % Interpolation zwischen Tabellenwerten
   end
end
%
% Darstellungsbereich
% ===================
ax1 = t2(1);
ax2 = t2(length(t2));
da  = ceil((10 + ax1 - ax2)/2);
ax1 = max([1,round(ax1 - da*(da>0))]);
ax2 = min([round(ax2 + da*(da>0)),t1(length(t1))]);
if ax2<ax1+5 ax1 = 1; end
ay1 = 1.2*min(x);
ay2 = 1.2*max(x);
%
% Darstellung
% ===========
h = fullscreen('Umtastung');
plot(t1,x,'or',t2,y,'b.','LineWidth',2.0)
axis([ax1,ax2,ay1,ay2])
title('Umtastung durch Shannon-Synthese','FontSize',15)
xlabel('Zeit');
ylabel('Signal');
legend('diskretes Eingangssignal','umgetastetes Ausgangssignal')
