function y = sinclean(x,fsoll,tol,typ)
% y = sinclean(x,fsoll,tol,typ)
% Das Signal wird mit einem Bandpass der Mitte fsoll und der 
% Bandbreite 2*tol gefiltert und seine Amplitude auf 1 gebracht.
%  x       :  komplexes (oder reelles) Inputsignal
%  fsoll   :  Sollfrequenz relativ zur Abtastfrequenz
%  tol     :  Toleranz fr die Sinusfrequenz
%  typ     :  legt die Art des Filters fest:
%  typ = 0 :  Multiraten-FIR-Filter
%  typ = 1 :  Butterworth-Filter
% Ausgegeben wird die komplexe Oszillatorschwingung y.
% Wenn y nicht auf eine Variable zugewiesen wird, gibt es 
% eine Graphik.

% Hinweis:  Das Ausgangssignal ist im Fall der Benutzung des 
% Butterworth-Filters wegen dessen nichtlinearen Phasenganges 
% nicht exakt in Phase mit dem Eingangssignal.

% Autor:  Klaus von der Heide;  e-mail:  v.d.heide@on-line.de

if nargin<4   typ = 0;     end
if nargin<3   tol = 0.02;  end
if nargin<2 fsoll = 0.05;  end
if nargin<1    
   t = 1:4000;
   x = exp(i*2*pi*0.055*t) + 0.3*randn(size(t));
else
   t = 1:length(x);
end

n = length(x);                    % Anzahl Abtastungen
t = 1:n;                          % Zeitbereich

s = exp(i*2*pi*fsoll*t);          % Sollsignal

u = x.*conj(s);                   % Transformation ins Basisband

if typ>0
   [b,a] = butter(2,2*tol);       % Basisbandfilter
   v     = filter(b,a,u);         % Filtern mit IIR-Filter
   d     = round(mean(grpdelay(b,a,1:50,50/tol))); % Filterverzgerung
else
   [koef,d] = mrprototype(2*tol); % Entwurf des Multiratenfilters
   [v,d] = mrfilter(koef,d,u);    % Filtern mit Multiratenfilter
end

v = v./abs(v);                    % Amplitude auf 1 setzen
m = min([length(s),length(v)]);   % Vektoren auf gleiche Lnge bringen:
t = 1:m; x = x(1:m); s = s(1:m); u = u(1:m); v = v(1:m);

y = v.*delay(s,d);                % Rcktransformation

if nargout>0 break; end           % wenn Zuweisung, dann keine Graphik

h1 = fullscreen('SinClean');
plot(t(1:m),[real(delay(x(1:m),d));real(delay(s(1:m),d));real(delay(u(1:m),d));...
             real(v(1:m));real(y(1:m))])
legend('Eingabe','Sollsignal','Basisbandsignal','TPsignal','Ausgabe')
set(h1,'KeyPressFcn','xzoomvis')

if typ>0  h2 = fullscreen('Frequenzgang'); filterplot({{b;a}}); end

sound(0.3*real(x),8000); pause(2)
sound(real(y(d:length(y))),8000)
