function z = sintrack(x,fsoll,bw1,typ1,bw2,typ2)
% z = sintrack(x,fsoll,bw1,typ1,bw2,typ2)
%  x          :  komplexes (oder reelles) Inputsignal
%  fsoll      :  Sollfrequenz relativ zur Abtastfrequenz
%  bw1        :  Fangbereich +-bw1
%  typ1       :  legt die Art des ersten Filters fest
%  bw2        :  bestimmt die Phasenreinheit des Ausgangssignals
%                und damit auch die Durchlaufverzgerung.
%                Sinvoll ist bw2<<bw1.
%  typ2       :  legt die Art des zweiten Filters fest:
%  typ1/2 = 0 :  Multiraten-FIR-Filter
%  typ1/2 = 1 :  Butterworth-Filter vom Grad 2
% Ausgegeben wird die komplexe Oszillatorschwingung z.
% Wenn z nicht auf eine Variable zugewiesen wird, gibt es 
% eine Graphik.
% Das hier eingesetzte einfache Multiratenfilter (mrprototype)
% erzeugt bei der Interpolation strende Alias. Deshalb ist das 
% Butterworth-Filter vorzuziehen.

% Das Prinzip von sintrack:
% Das Eingangssignal x wird um fsoll ins Basisband verschoben und 
% dort mit einem Tiefpass der Bandbreite bw1 gefiltert. 
% Dieses Filter bestimmt den Fangbereich von sintrack. 
% Die Phase p(t) des gefilterten Signals wird mit einem weiteren 
% Tiefpassfilter der Bandbreite bw2 gefiltert mit dem Resultat 
% w(t). Dieses zweite Filter bestimmt das Spektrum des Ausgabe-
% signals z und damit, welchen nderungen des Eingangssignals  x 
% das Ausgangssignal z noch folgen kann. z wird synthetisiert.

% Autor:  Klaus von der Heide;  e-mail:  v.d.heide@on-line.de

if nargin<6  typ2 = 1;     end
if nargin<5   bw2 = 0.001; end
if nargin<4  typ1 = 1;     end
if nargin<3   bw1 = 0.08;  end
if nargin<2 fsoll = 0.1;   end
if nargin<1    
   t = 1:8000;
   x = exp(i*(2*pi*0.1*t+100*sin(2*pi*0.0005*t)+0.00002*t.*t)) + randn(size(t));
else
   t = 1:length(x);
end

n = length(x);                      % Anzahl Abtastungen
t = 1:n;                            % Zeitbereich

u = x.*exp(-i*2*pi*fsoll*t);        % Transformation ins Basisband

if typ1>0
   [b1,a1] = butter(2,2*bw1);       % Basisbandfilter
   v  = filter(b1,a1,u);            % Filtern mit IIR-Filter
   d1 = round(mean(grpdelay(b1,a1,1:50,50/bw1))); % Filterverzgerung
else
   [koef1,dz1] = mrprototype(2*bw1);% Entwurf des Multiratenfilters
   [v,d1] = mrfilter(koef1,dz1,u);  % Filtern mit Multiratenfilter
end

av= angle(v);
av(1:d1) = 0;                       % Einschwingvorgang lschen
p = unwrap(av);                     % Phasenwinkel p

if typ2>0
   [b2,a2] = butter(2,2*bw2);       % Phasenfilter
   w  = filter(b2,a2,p);            % Phase filtern mit IIR-Filter
   d2 = round(mean(grpdelay(b2,a2,1:50,50/bw2))); % Filterverzgerung
else
   [koef2,dz2] = mrprototype(2*bw2);% Entwurf des Multiratenfilters
   [w,d2] = mrfilter(koef2,dz2,p);  % Filtern mit Multiratenfilter
end

m = min([length(u),length(w)]); 
t = t(1:m);

y = exp(i*(2*pi*fsoll*(t-(d1+d2)) + w(t))); % gesubertes Signal

r = delay(x(t),d1+d2).*conj(y); 

if typ2>0
   q = filter(b2,a2,r);             % Filtern mit IIR-Filter
else
   q = mrfilter(koef2,dz2,r);       % Filtern mit Multiratenfilter
end
q(1:d1+d2+d2) = 0;                  % Einschwingvorgang lschen
psi = unwrap(angle(q));             % Phasenverschiebung zwische x und y
m = min([length(x),length(q)]); 
t = t(1:m);

z = exp(i*(2*pi*fsoll*(t-(d1+d2+d2)) + delay(w(t),d2) + psi(t))); % Synthese

%============================================================================

if nargout>0 break; end

v = v(1:m);
p = p(1:m);
w = w(1:m);
h = fullscreen('SinTrack');
plot(t,[real(delay(x,d1+d2+d2));real(delay(u,d1+d2+d2));delay(real(v),d2+d2);...
        delay(p,d2+d2);delay(w,d2);real(delay(y,d2));psi;real(z)])
legend('Eingabe','Basisbandsignal','TP1-Signal','Phase','TP2-Phase','y','Psi','Ausgabe')
set(h,'KeyPressFcn','xzoomvis')

sound(0.3*real(x),8000); pause(2)
sound(real(exp(i*2*pi*fsoll*t).*v),8000); pause(2)
sound(real(z(d1+d2+d2:length(z))),8000)
