function x0 = umtastdemo(x,k,n,r,s,p)
% x0 = umtastdemo(x,k,n,r,s,p)
% Asynchrone Umtastung des Signals x und erneute Umtastung zurck 
% in die ursprngliche Taktung. Die Differenz nach zweimaliger 
% Umtastung wird dargestellt sowie alle Signale direkt.
%
% Die Parameter k, n, r werden direkt an die Funktion resample bergeben:
% k  :  Halbe Fensterbreite fr die sinc-Funktion
% n  :  Anzahl der Sttzstellen pro Abtastschritt fr die sinc-Tabelle
% r  :  CosRollOff-Faktor fr das Fenster der sinc-Funktion
% s  :  Schrittweite der neuen Abtastung (Schritt der ursprnglichen = 1)
% p  :  Zeitversatz der ersten neuen Abtastung
% x0 :  Standardabweichung zwischen ursprnglichem Signal und zweiter Umtastung.
% 
% Wenn  s > 1 ist, muss das Spektrum von x auf den Bereich 
% 0...1/s beschrnkt sein.
%
% Beispiele: 
% umtastdemo(randn(1,500),100,16,0,0.653172,0.1)
% [b,a] = ellip(6,1,60,0.1);  
% umtastdemo(filter(b,a,randn(1,500)),100,16,0,5/3,0)
% ellip(6,1,60,0.3);
% umtastdemo(filter(b,a,randn(1,4000)),585,14,0,0.5971,0)

% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de
%
% Konstanten und Defaults
% =======================
if nargin<6  p = 0.7;   end
if nargin<5  s = 1.181; end
if nargin<4  r = 0;     end
if nargin<3  n = 8;     end
if nargin<2  k = 20;    end
if nargin<1  
   [u,v] = ellip(6,1,60,0.1);
   x = filter(u,v,randn(1,200)); 
end
if length(x)<4*k+10 error('Es muss gelten: Signallnge > 4*k + 10'); end
fsiz = 12;                        % FontSize
m    = length(x);                 % Lnge des Signals (Abtastpunkte)
%
% Signal und Umtastung
% ====================
t1  = 1:length(x);                % ursprngliche Abtastung
t2  = 1+p:s:length(x);            % Zeitpunkte der Neuabtastung
y   = resample(x,t1,t2,k,r,n);    % erste Umtastung
z   = resample(y,t2,t1,k,r,n);    % Umtastung zurck auf ursprngliche
%
% Standardabweichung des Fehlers
% ==============================
ff = z - x;                       % Differenz
zz = ff(1+2*k:length(z)-2*k);     % Differenz im korrekten Zeitbereich
x0 = sqrt(zz*zz'/length(zz));     % Standardabweichung
%
% Darstellung
% ===========
h = fullscreen('Fehler der Umtastung');
subplot(2,1,1);                   % Darstellung der Umtastung
plot(t1,x,'or',t1,z,'b.',t2,y,'gx');
axis([1+2*k,length(x)-2*k,1.1*min([x,y,z]),1.1*max([x,y,z])]);
line(t2,y,'LineStyle','none','Marker','x','Color','g','LineWidth',2.0);
title('Eingabeabtastung, Umtastung und zweite Umtastung','FontSize',16)
legend('Eingabeabtastung','zweite Umtastung','erste Umtastung')

subplot(2,1,2)                    % Differenz zwischen ursprnglichen und
plot(t1,ff,'or','LineWidth',2.0)  % zweimal neu abgetsteten Werten
set(gca,'xlim',[1+2*k,length(x)-2*k]);
title('Fehler nach zweimaliger Umtastung','FontSize',16)
xlabel('Zeit','FontSize',fsiz);
ylabel('Fehler','FontSize',fsiz);

