function  y = umtastung(x,t1,t2,b,n)
% y = umtastung(x,t1,t2,b,n)
% Interpolation des diskreten Signals x durch Zerlegung in interpolierte 
% Impulsantworten des durch die Filterkoeffizienten b gegebenen Filters.
% Wenn die Ausgabe nicht auf eine Variable zugewiesen wird, gibt es eine
% Graphik mit den Abtastwerten von Eingangs- und umgetasteten Ausgangssignal.
%
%  x      :   Eingabesamples
%  t1, t2 :   Vektoren der quidistanten Zeitmarken fr die Abtastung 
%             von x bzw. der zu berechnenden Ausgabe.
%  b      :   Filterkoeffizienten
%  n      :   Zahl der Unterteilungen des Zeitintervalls zwischen zwei 
%             Abtastungen von t1 in der Tabelle der Impulsantwort.
%
% Beispiele: 
% [u,v]=ellip(6,0.1,60,0.6); b=fir1(30,0.8); x=filter(u,v,randn(1,200)); 
% umtastung(x,1:length(x),100+(1:1/11:50),b,8);    % synchrone Hochtastung
% umtastung(x,1:length(x),100+(1:1/11:50),b,1);    % Tabelle zu klein
% umtastung(x,1:length(x),100+(1.11:pi/7:50),b,8); % asynchrone Hochtastung
% umtastung(x,1:length(x),100+(1.11:pi/3:50),b,8); % asynchrone Umtastung
% [u,v]=ellip(6,0.1,60,0.6); b=fir1(8,0.8); x=filter(u,v,randn(1,200)); 
% umtastung(x,1:length(x),100+(1:1/11:50),b,8);    % zu wenig Koeffizienten bei b
% [u,v]=ellip(6,0.1,60,0.6); b=fir1(30,0.4); x=filter(u,v,randn(1,200));
% umtastung(x,1:length(x),100+(1:1/11:50),b,8);    % falsche Durchlassbereiche

% Autor: K. v. d. Heide, e-mail:  v.d.heide@on-line.de
%
% Konstanten und Defaults
% =======================
if nargin<1   % ein Beispiel
   [u,v] = ellip(6,0.1,60,0.5); 
   b  = fir1(30,0.8); 
   x  = filter(u,v,randn(1,50));
   x  = x(16:39);
   t1 = 16:39;
   t2 = (1:27)/1.181+15.7;
   n  = 8;
end
m1 = length(x);
if m1~= length(t1)  
   error('Input x und dessen Zeitmarken mssen dieselbe Lnge haben.');
end
m2 = length(t2);
mb = length(b);
k  = (length(b)-1)/2;
dt1= t1(2)-t1(1);
x0 = [zeros(1,k+1),x,zeros(1,k+1)];

% Tabelle fr die Impulsantwort erstellen
% =======================================
bb = [0,b,0];
tb = interp1((1:mb+2),bb,(0:n*(mb+2)-1)/n,'cubic');
tb = [tb(n+1:n*(mb+2)),0];
mt = length(tb);
%
% Umtastung
% =========
for i = 1:m2                                % Schleife ber alle Ausgabeabtastungen
   tt2 = t2(i);                             % fr diesen Zeitpunkt jetzt berechnen
%   tt1 = max(t1.*(t1<=tt2));                % der letzte Abtastpunkt t1(j) vor t2(i)
%   j   = max((1:m1).*(tt1==t1))+k+1;        % Index j des letzten x(j) vor t2(i)
   j   = min(find(t1<=tt2&tt2<t1+dt1));     % der letzte Abtastpunkt t1(j) vor t2(i)
   if isempty(j)
      y(i) = 0;
   else
      tt1 = t1(j);
      j   = j + k + 1;
      xx  = (tt2 - tt1)/dt1*n;
      x1  = floor(xx);                      % Offset fr Tabelle
      x2  = xx - x1;                        % Offset fr Interpolation in Tabelle
      aa  = tb(x1+1:n:mt-1)*x0(j+k+1:-1:j-k)'; % erster Tabellenwert
      bb  = tb(x1+2:n:mt)*x0(j+k+1:-1:j-k)';   % zweiter Tabellenwert
      y(i)= aa + (bb - aa)*x2;              % Interpolation zwischen Tabellenwerten
   end
end
%
if nargout>0  break; end                    % keine Graphik wenn Zuweisung auf y
%
% Darstellungsbereich
% ===================
ax1 = t2(1) + mb/2;
ax2 = t2(length(t2)) - mb/2;
da  = ceil((10 + ax1 - ax2)/2);
ax1 = max([1,round(ax1 - da*(da>0))]);
ax2 = min([round(ax2 + da*(da>0)),t1(length(t1))]);
if ax2<ax1+5 ax1 = 1; end
ay1 = 1.2*min(x);
ay2 = 1.2*max(x);
%
% Darstellung
% ===========
h = fullscreen('Umtastung');
plot(t1,x,'or',t2,y,'b.','LineWidth',2.0)
axis([ax1,ax2,ay1,ay2])
title('Umtastung durch Synthese aus der Interpolation der Impulsantwort b','FontSize',15)
xlabel('Zeit');
ylabel('Signal');
legend('diskretes Eingangssignal','umgetastetes Ausgangssignal')
