function winzeitfreq(sigstr,fs,T,win,lim)
% winzeitfreq(sigstr,fs,T,win,lim)
% winzeitfreq tastet das im String  sigstr  als MATLAB-Expression 
% angegebene Signal im Zeitbereich von 0 ... T  mit der
% Abtastrate  fs  ab und berechnet die diskrete Fouriertransformation.
% Die Zeit ist als Variable mit dem Identifier  t  fest vorgegeben.
% Das Signal wird im oberen Bild im Zeitbereich gezeichnet.
% Darunter wird die spektrale Leistungsdichte mit n/2 Punkten in dB geplottet.
%
% sigstr :  String zur Definition des Signals mit der Zeitvariablen t
%           Alternativ kann ein Vektor von Abtastwerten angegeben werden.
%           Auch in diesem Fall gelten die Angaben fs und T.  Evtl. werden 
%           dem Abtastvektor Nullen angefgt oder Elemente fortgelassen.
% fs     :  Abtastrate
% T      :  Der Zeitbereich ist  0 <= t <= T
% win    :  Angabe zur Fensterung des Signals. win kann entweder einer 
%           der folgenden Strings sein: 
%           'bartlett', 'blackman', boxcar', 'hamming', 'hanning', 'kaiser', 'triang'
%           Im Falle von 'chebwin' wird r = 71.5 fr 71.5 dB genommen.
%           Im Falle von 'kaiser' wird Beta = 7.8573 fr 80 dB genommen.
%           Oder win ist eine Zahl. Dann gibt win die Absenkung der Artefakte 
%           in dB an, die mit dem Kaiserfenster erreicht werden soll.
% lim    :  explizite Angabe des Darstellungsbereiches fr die FFT in dB.

% Autor: K. v. d. Heide, e-mail:  v.d.heide@on-line.de
%
% Defaults
% ========
if nargin<3  T  = 2;  end
if nargin<2  fs = 30; end
if nargin<1  sigstr = 'sin(2*pi*3.1*t)+0.7*sin(2*pi*5.3*t)'; end
fsiz = 12;                        % FontSize
n  = round(T*fs);                 % Anzahl der Abtastpunkte im Zeitfenster                     
fenster = ones(1,n);
fstr = '  boxcar';
if nargin > 3
   if ischar(win)
      fstr = ['  ',win];   
      switch win
         case 'bartlett', fenster = bartlett(n)';
         case 'blackman', fenster = blackman(n)';
         case 'boxcar',   fenster = boxcar(n)';
         case 'chebwin'
            fenster = chebwin(n,71.5)';
            fstr = [fstr,' ( r = 71.5)'];
         case 'hamming',  fenster = hamming(n)';
         case 'hanning',  fenster = hanning(n)';
         case 'kaiser'
            fenster = kaiser(n,7.8573)';
            fstr = [fstr,' ( beta = 7.8573)'];
         case 'triang',   fenster = triang(n)';
         otherwise fenster = hamming(n)';
      end
   else
      fstr = '  kaiser';
      if win < 21                 % optimales beta fr das Kaiserfenster
         beta = 0;                % berechnen
         fstr = '  boxcar';
      elseif win < 50
         beta = 0.5842*(win-21)^0.4 + 0.07886*(win-21);
         fstr = [fstr,sprintf(' ( beta = %2.4f)',beta)];
      else
         beta = 0.1102*(win-8.7);
         fstr = [fstr,sprintf(' ( beta = %2.4f)',beta)];
      end;
      fenster = kaiser(n,beta)';  % Kaiserfenster
   end
end;
%
% Signal und Abtastung
% ====================
if ischar(sigstr) 
   tt = T*(0:799)/800;            % "kontinuierlicher Zeitbereich"
   t  = tt;
   ss = eval(sigstr);             % "kontinuierliches Signal"
   t  = T*(0:n-1)/n;              % Abtastpunkte
   s  = eval(sigstr);             % Abtastung des Signals
   fktstr = sigstr;
else
   s  = [sigstr(1:min([n,length(sigstr)])),zeros(1,n-length(sigstr))];
   ss = s;
   t  = T*(0:n-1)/n;
   tt = t;
   fktstr = 'Samples';
end
%
% FFT
% ===
S  = fft(s.*fenster);             % FFT gefenstert
y  = S.*conj(S)/n;                % spektrale Leistungsdichte
ly = 10*log10(y);                 % dies in dB
f  = (0:n-1)/T;                   % Frequenzbereich
%
% Darstellung
% ===========
h = fullscreen('Diskrete Fouriertransformation');
subplot(2,1,1);                   % zweiteiligen Plot festlegen
plot(tt,ss);                      % Plot Zeitbereich: kontinuierlich
line([0,T],[0,0]);                % Null-Linie
for k=1:n  line([t(k),t(k)],[0,s(k)],'Color','r'); end
line(t,s,'Marker','.','Color','r','LineStyle','none','MarkerSize',15);
title(sprintf(['Signal:  ',fktstr,'  (eine Periode)  abgetastet mit  fs =%7.2f Hz'],fs),...
      'FontSize',fsiz);
xlabel('t [sec]');
set(gca,'xlim',[0,T])
yl = get(gca,'ylim'); set(gca,'ylim',1.1*yl)
line(t,fenster*yl(2),'Marker','.','Color',0.7*[1 1 1],'LineStyle','none')
subplot(2,1,2);
plot(f,ly);                       % Plot Frequenzbereich
if nargin>4 set(gca,'ylim',lim); end
yl = get(gca,'ylim');
for k=1:n  line([f(k),f(k)],[yl(1),ly(k)],'Color','g'); end
line(f,ly,'Marker','.','Color','g','LineStyle','none','MarkerSize',15);
title(strcat('Spektrale Leistungsdichte  (eine Periode)    Fenster:',fstr),...
      'FontSize',fsiz);
xlabel('Frequenz [Hz]','FontSize',fsiz);
ylabel('Dichte in dB','FontSize',fsiz);
set(gca,'xlim',[0,f(n)])
